# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-09-29 09:48
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0071_remove_old_image_hashes'),
    ]

    operations = [
        migrations.AddField(
            model_name='sitesettings',
            name='fb_analytics_app_id',
            field=models.CharField(blank=True, help_text='The tracking ID to be used to view Facebook Analytics', max_length=25, null=True, verbose_name='Facebook Analytics App ID'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-05-17 16:42
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('profiles', '0001_squashed_0021_remove_uuid_null'),
    ]

    operations = [
        migrations.AddField(
            model_name='userprofile',
            name='auth_service_uuid',
            field=models.UUIDField(null=True, unique=True),
        ),
    ]



from django.utils import timezone

import re

from django import forms
from django.forms.extras.widgets import SelectDateWidget
from django.contrib.auth import authenticate
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import AuthenticationForm
from django.utils.translation import ugettext_lazy as _
from django.conf import settings

from wagtail.wagtailcore.models import Site
from molo.profiles.models import UserProfile, UserProfilesSettings

from phonenumber_field.formfields import PhoneNumberField

User = get_user_model()

REGEX_PHONE = settings.REGEX_PHONE if hasattr(settings, 'REGEX_PHONE') else \
    r'.*?(\(?\d{3})? ?[\.-]? ?\d{3} ?[\.-]? ?\d{4}.*?'

REGEX_EMAIL = settings.REGEX_EMAIL if hasattr(settings, 'REGEX_PHONE') else \
    r'([\w\.-]+@[\w\.-]+)'


class MoloAuthenticationForm(AuthenticationForm):

    def clean(self):
        username = self.cleaned_data.get('username')
        password = self.cleaned_data.get('password')

        if username and password:
            self.user_cache = authenticate(request=self.request,
                                           username=username,
                                           password=password)
            if self.user_cache is None:
                raise forms.ValidationError(
                    self.error_messages['invalid_login'],
                    code='invalid_login',
                    params={'username': self.username_field.verbose_name},
                )
            else:
                self.confirm_login_allowed(self.user_cache)

        return self.cleaned_data


def get_validation_msg_fragment():
    site = Site.objects.get(is_default_site=True)
    profile_settings = UserProfilesSettings.for_site(site)

    invalid_msg = ''

    if getattr(profile_settings, 'prevent_email_in_username', False) \
            and getattr(profile_settings, 'prevent_phone_number_in_username',
                        False):
        invalid_msg = 'phone number or email address'

    elif getattr(profile_settings, 'prevent_phone_number_in_username', False):
        invalid_msg = 'phone number'

    elif getattr(profile_settings, 'prevent_email_in_username', False):
        invalid_msg = 'email address'

    return invalid_msg


def validate_no_email_or_phone(input):
    site = Site.objects.get(is_default_site=True)
    profile_settings = UserProfilesSettings.for_site(site)

    regexes = []
    if profile_settings.prevent_phone_number_in_username:
        regexes.append(REGEX_PHONE)

    if profile_settings.prevent_email_in_username:
        regexes.append(REGEX_EMAIL)

    for regex in regexes:
        match = re.search(regex, input)
        if match:
            return False

    return True


class RegistrationForm(forms.Form):
    username = forms.RegexField(
        regex=r'^[\w.@+-]+$',
        widget=forms.TextInput(
            attrs=dict(
                required=True,
                max_length=30,
            )
        ),
        label=_("Username"),
        error_messages={
            'invalid': _("This value must contain only letters, "
                         "numbers and underscores."),
        }
    )
    password = forms.RegexField(
        regex=r'^\d{4}$',
        widget=forms.PasswordInput(
            attrs=dict(
                required=True,
                render_value=False,
                type='password',
            )
        ),
        max_length=4,
        min_length=4,
        error_messages={
            'invalid': _("This value must contain only numbers."),
        },
        label=_("PIN")
    )
    email = forms.EmailField(required=False)
    mobile_number = PhoneNumberField(required=False)
    terms_and_conditions = forms.BooleanField(required=True)
    alias = forms.CharField(
        label=_("Display Name"),
        required=False
    )
    date_of_birth = forms.DateField(
        widget=SelectDateWidget(
            years=list(reversed(range(1930, timezone.now().year + 1)))
        ),
        required=False
    )
    gender = forms.CharField(
        label=_("Gender"),
        required=False
    )
    location = forms.CharField(
        label=_("Location"),
        required=False
    )
    education_level = forms.CharField(
        label=_("Education Level"),
        required=False
    )
    next = forms.CharField(required=False)

    def __init__(self, *args, **kwargs):
        questions = kwargs.pop("questions", [])
        request = kwargs.pop("request", [])
        super(RegistrationForm, self).__init__(*args, **kwargs)
        if not request:
            site = Site.objects.get(is_default_site=True)
            profile_settings = UserProfilesSettings.for_site(site)
        else:
            profile_settings = UserProfilesSettings.for_site(request.site)
        self.fields['mobile_number'].required = (
            profile_settings.mobile_number_required and
            profile_settings.show_mobile_number_field and
            profile_settings.country_code)
        self.fields['email'].required = (
            profile_settings.email_required and
            profile_settings.show_email_field)
        self.fields['alias'].required = (
            profile_settings.activate_display_name and
            profile_settings.capture_display_name_on_reg and
            profile_settings.display_name_required)
        self.fields['date_of_birth'].required = (
            profile_settings.activate_dob and
            profile_settings.capture_dob_on_reg and
            profile_settings.dob_required)
        self.fields['gender'].required = (
            profile_settings.activate_gender and
            profile_settings.capture_gender_on_reg and
            profile_settings.gender_required)
        self.fields['location'].required = (
            profile_settings.activate_location and
            profile_settings.capture_location_on_reg and
            profile_settings.location_required)
        self.fields['education_level'].required = (
            profile_settings.activate_education_level and
            profile_settings.capture_education_level_on_reg and
            profile_settings.activate_education_level_required)

        # Security questions fields are created dynamically.
        # This allows any number of security questions to be specified
        for index, question in enumerate(questions):
            self.fields["question_%s" % index] = forms.CharField(
                label=question.title,
                widget=forms.TextInput(
                    attrs=dict(
                        max_length=150,
                    )
                )
            )
            self.fields["question_%s" % index].required = (
                profile_settings.show_security_question_fields and
                profile_settings.security_questions_required
            )

    def security_questions(self):
        return [
            self[name] for name in filter(
                lambda x: x.startswith('question_'), self.fields.keys()
            )
        ]

    def clean_username(self):
        validation_msg_fragment = get_validation_msg_fragment()

        if User.objects.filter(
                username__iexact=self.cleaned_data['username']
        ).exists():
            raise forms.ValidationError(_("Username already exists."))

        if not validate_no_email_or_phone(self.cleaned_data['username']):
            raise forms.ValidationError(
                _(
                    "Sorry, but that is an invalid username. Please don't use"
                    " your %s in your username." % validation_msg_fragment
                )
            )

        return self.cleaned_data['username']

    def is_valid(self):
        if 'mobile_number' in self.data:
            if not self.data['mobile_number'].startswith('+'):
                site = Site.objects.get(is_default_site=True)
                profile_settings = UserProfilesSettings.for_site(site)
                number = self.data['mobile_number']
                if number:
                    if number.startswith('0'):
                        number = number[1:]
                    number = profile_settings.country_code + \
                        number
                self.data = self.data.copy()
                self.data['mobile_number'] = number
        valid = super(RegistrationForm, self).is_valid()
        return valid

    def clean_alias(self):
        validation_msg_fragment = get_validation_msg_fragment()

        alias = self.cleaned_data['alias']

        if not validate_no_email_or_phone(alias):
            raise forms.ValidationError(
                _(
                    "Sorry, but that is an invalid display name. "
                    "Please don't use your %s in your display name."
                    % validation_msg_fragment
                )
            )

        return alias


class DoneForm(forms.Form):
    date_of_birth = forms.DateField(
        widget=SelectDateWidget(
            years=list(reversed(range(1930, timezone.now().year + 1)))
        )
    )
    alias = forms.CharField(
        label=_("Display Name"),
        required=False
    )
    gender = forms.CharField(
        label=_("Gender"),
        required=False
    )
    location = forms.CharField(
        label=_("Location"),
        required=False
    )
    education_level = forms.CharField(
        label=_("Education Level"),
        required=False
    )

    def __init__(self, *args, **kwargs):
        request = kwargs.pop("request", [])
        super(DoneForm, self).__init__(*args, **kwargs)
        if not request:
            site = Site.objects.get(is_default_site=True)
            profile_settings = UserProfilesSettings.for_site(site)
        else:
            profile_settings = UserProfilesSettings.for_site(request.site)
        self.fields['date_of_birth'].required = (
            profile_settings.activate_dob and not
            profile_settings.capture_dob_on_reg and
            profile_settings.dob_required)
        self.fields['alias'].required = (
            profile_settings.activate_display_name and not
            profile_settings.capture_display_name_on_reg and
            profile_settings.display_name_required)
        self.fields['gender'].required = (
            profile_settings.activate_gender and not
            profile_settings.capture_gender_on_reg and
            profile_settings.gender_required)
        self.fields['location'].required = (
            profile_settings.activate_location and not
            profile_settings.capture_location_on_reg and
            profile_settings.location_required)
        self.fields['education_level'].required = (
            profile_settings.activate_education_level and not
            profile_settings.capture_education_level_on_reg and
            profile_settings.activate_education_level_required)


class EditProfileForm(forms.ModelForm):
    alias = forms.CharField(
        label=_("Display Name"),
        required=False
    )
    date_of_birth = forms.DateField(
        widget=SelectDateWidget(
            years=list(reversed(range(1930, timezone.now().year + 1)))
        ),
        required=False
    )
    gender = forms.CharField(
        label=_("Gender"),
        required=False
    )
    location = forms.CharField(
        label=_("Location"),
        required=False
    )
    education_level = forms.CharField(
        label=_("Education Level"),
        required=False
    )
    mobile_number = PhoneNumberField(required=False)
    email = forms.EmailField(required=False)

    def __init__(self, *args, **kwargs):
        request = kwargs.pop("request", [])
        super(EditProfileForm, self).__init__(*args, **kwargs)
        if not request:
            site = Site.objects.get(is_default_site=True)
            profile_settings = UserProfilesSettings.for_site(site)
        else:
            profile_settings = UserProfilesSettings.for_site(request.site)
        self.fields['mobile_number'].required = (
            profile_settings.mobile_number_required and
            profile_settings.show_mobile_number_field and
            profile_settings.country_code)
        self.fields['email'].required = (
            profile_settings.email_required and
            profile_settings.show_email_field)
        self.fields['alias'].required = (
            profile_settings.activate_display_name and
            profile_settings.display_name_required)
        self.fields['date_of_birth'].required = (
            profile_settings.activate_dob and
            profile_settings.dob_required)
        self.fields['gender'].required = (
            profile_settings.activate_gender and
            profile_settings.gender_required)
        self.fields['location'].required = (
            profile_settings.activate_location and
            profile_settings.location_required)
        self.fields['education_level'].required = (
            profile_settings.activate_education_level and
            profile_settings.activate_education_level_required)

    class Meta:
        model = UserProfile
        fields = ['alias', 'date_of_birth', 'mobile_number',
                  'gender', 'location', 'education_level']

    def clean_alias(self):
        validation_msg_fragment = get_validation_msg_fragment()

        alias = self.cleaned_data['alias']

        if not validate_no_email_or_phone(alias):
            raise forms.ValidationError(
                _(
                    "Sorry, but that is an invalid display name. "
                    "Please don't use your %s in your display name."
                    % validation_msg_fragment
                )
            )

        return alias

    def is_valid(self):
        if 'mobile_number' in self.data:
            if not self.data['mobile_number'].startswith('+'):
                site = Site.objects.get(is_default_site=True)
                profile_settings = UserProfilesSettings.for_site(site)
                number = self.data['mobile_number']
                if number:
                    if number.startswith('0'):
                        number = number[1:]
                    number = profile_settings.country_code + \
                        number
                self.data = self.data.copy()
                self.data['mobile_number'] = number
        valid = super(EditProfileForm, self).is_valid()
        return valid


class ProfilePasswordChangeForm(forms.Form):
    old_password = forms.RegexField(
        regex=r'^\d{4}$',
        widget=forms.PasswordInput(
            attrs=dict(
                required=True,
                render_value=False,
                type='password',
            )
        ),
        max_length=4, min_length=4,
        error_messages={'invalid': _("This value must contain only  \
         numbers.")},
        label=_("Old Password")
    )
    new_password = forms.RegexField(
        regex=r'^\d{4}$',
        widget=forms.PasswordInput(
            attrs=dict(
                required=True,
                render_value=False,
                type='password',
            )
        ),
        max_length=4,
        min_length=4,
        error_messages={'invalid': _("This value must contain only  \
         numbers.")},
        label=_("New Password")
    )
    confirm_password = forms.RegexField(
        regex=r'^\d{4}$',
        widget=forms.PasswordInput(
            attrs=dict(
                required=True,
                render_value=False,
                type='password',
            )
        ),
        max_length=4,
        min_length=4,
        error_messages={
            'invalid': _("This value must contain only numbers."),
        },
        label=_("Confirm Password")
    )

    def clean(self):
        new_password = self.cleaned_data.get('new_password', None)
        confirm_password = self.cleaned_data.get('confirm_password', None)
        if (new_password and confirm_password and
                (new_password == confirm_password)):
            return self.cleaned_data
        else:
            raise forms.ValidationError(_('New passwords do not match.'))


class ForgotPasswordForm(forms.Form):
    username = forms.RegexField(
        regex=r'^[\w.@+-]+$',
        widget=forms.TextInput(
            attrs=dict(
                required=True,
                max_length=30,
            )
        ),
        label=_("Username"),
        error_messages={
            'invalid': _("This value must contain only letters, "
                         "numbers and underscores."),
        }
    )

    def __init__(self, *args, **kwargs):
        questions = kwargs.pop("questions", [])
        super(ForgotPasswordForm, self).__init__(*args, **kwargs)

        for index, question in enumerate(questions):
            self.fields["question_%s" % index] = forms.CharField(
                label=question.title,
                widget=forms.TextInput(
                    attrs=dict(
                        required=True,
                        max_length=150,
                    )
                )
            )


class ResetPasswordForm(forms.Form):
    username = forms.CharField(
        widget=forms.HiddenInput()
    )

    token = forms.CharField(
        widget=forms.HiddenInput()
    )

    password = forms.RegexField(
        regex=r'^\d{4}$',
        widget=forms.PasswordInput(
            attrs=dict(
                required=True,
                render_value=False,
                type='password',
            )
        ),
        max_length=4,
        min_length=4,
        error_messages={
            'invalid': _("This value must contain only numbers."),
        },
        label=_("PIN")
    )

    confirm_password = forms.RegexField(
        regex=r'^\d{4}$',
        widget=forms.PasswordInput(
            attrs=dict(
                required=True,
                render_value=False,
                type='password',
            )
        ),
        max_length=4,
        min_length=4,
        error_messages={
            'invalid': _("This value must contain only numbers."),
        },
        label=_("Confirm PIN")
    )



from django.test import TestCase
from molo.core import utils
from molo.core.tests.base import MoloTestCaseMixin
from molo.core.models import SiteLanguageRelation, Languages, Main
from molo.core.utils import generate_slug


class TestUtils(TestCase, MoloTestCaseMixin):

    def test_get_locale_code(self):
        self.assertEquals(utils.get_locale_code(), 'en')
        self.assertEquals(utils.get_locale_code('en-GB'), 'en-GB')
        self.assertEquals(utils.get_locale_code('en_GB'), 'en-GB')
        self.assertEquals(utils.get_locale_code('fr_FR'), 'fr-FR')
        self.assertEquals(utils.get_locale_code('zu-ZA'), 'zu-ZA')
        self.assertEquals(utils.get_locale_code('en'), 'en')

    def test_slugify(self):
        self.mk_main()
        main = Main.objects.all().first()
        self.language_setting = Languages.objects.create(
            site_id=main.get_site().pk)
        self.english = SiteLanguageRelation.objects.create(
            language_setting=self.language_setting,
            locale='en',
            is_active=True)

        self.mk_section(self.main, title='Your mind')
        self.assertEquals(generate_slug('Your mind'), 'your-mind-1')

        self.mk_section(self.main, title='Your mind')
        self.assertEquals(generate_slug('Your mind'), 'your-mind-2')

        self.assertEquals(generate_slug(None), 'no-title')



from django.core.cache import cache
from django.forms.utils import pretty_name
from django.utils.html import format_html
from wagtail.wagtailadmin.edit_handlers import EditHandler

from itertools import chain

from django.utils import timezone as django_timezone
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import get_language_from_request
from django.shortcuts import redirect
from django.db.models.signals import (pre_save, post_save)
from django.dispatch import receiver, Signal
from django.template.response import TemplateResponse

from taggit.models import TaggedItemBase
from modelcluster.fields import ParentalKey
from modelcluster.tags import ClusterTaggableManager
from modelcluster.models import ClusterableModel

from wagtail.contrib.settings.models import BaseSetting, register_setting
from wagtail.wagtailcore.models import Page, Orderable, Site
from wagtail.wagtailcore.fields import StreamField
from wagtail.wagtailsearch import index
from wagtail.wagtailadmin.edit_handlers import (
    FieldPanel, FieldRowPanel, StreamFieldPanel, PageChooserPanel,
    MultiFieldPanel, InlinePanel)
from wagtail.wagtailimages.edit_handlers import ImageChooserPanel
from wagtail.wagtailcore import blocks
from wagtail.wagtailcore.models import PageManager
from wagtail.wagtailimages.blocks import ImageChooserBlock
from wagtail.wagtailimages.models import Image
from wagtail.contrib.wagtailroutablepage.models import route, RoutablePageMixin
from wagtailmedia.blocks import AbstractMediaChooserBlock
from wagtailmedia.models import AbstractMedia
from wagtail.wagtailcore.signals import page_unpublished

from molo.core.blocks import MarkDownBlock, SocialMediaLinkBlock
from molo.core import constants
from molo.core.api.constants import ERROR
from molo.core.forms import ArticlePageForm
from molo.core.utils import get_locale_code, generate_slug
from molo.core.mixins import PageEffectiveImageMixin
from molo.core.molo_wagtail_models import MoloPage
from molo.core.utils import (
    separate_fields,
    add_json_dump,
    add_list_of_things,
    attach_image,
    add_stream_fields,
    get_image_hash
)


class BaseReadOnlyPanel(EditHandler):
    def render(self):
        value = getattr(self.instance, self.attr)
        if callable(value):
            value = value()
        return format_html('<div style="padding-top: 1.2em;">{}</div>', value)

    def render_as_object(self):
        return format_html(
            '<fieldset><legend>{}</legend>'
            '<ul class="fields"><li><div class="field">{}</div></li></ul>'
            '</fieldset>',
            self.heading, self.render())

    def render_as_field(self):
        return format_html(
            '<div class="field">'
            '<label>{}{}</label>'
            '<div class="field-content">{}</div>'
            '</div>',
            self.heading, _(':'), self.render())


class ReadOnlyPanel:
    def __init__(self, attr, heading=None, classname=''):
        self.attr = attr
        self.heading = pretty_name(self.attr) if heading is None else heading
        self.classname = classname

    def bind_to_model(self, model):
        return type(str(_('ReadOnlyPanel')), (BaseReadOnlyPanel,),
                    {'attr': self.attr, 'heading': self.heading,
                     'classname': self.classname})


@register_setting
class SiteSettings(BaseSetting):
    logo = models.ForeignKey(
        'wagtailimages.Image',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+'
    )
    ga_tag_manager = models.CharField(
        verbose_name=_('Local GA Tag Manager'),
        max_length=255,
        null=True,
        blank=True,
        help_text=_(
            "Local GA Tag Manager tracking code (e.g GTM-XXX) to be used to "
            "view analytics on this site only")
    )
    global_ga_tag_manager = models.CharField(
        verbose_name=_('Global GA Tag Manager'),
        max_length=255,
        null=True,
        blank=True,
        help_text=_(
            "Global GA Tag Manager tracking code (e.g GTM-XXX) to be used"
            " to view analytics on more than one site globally")
    )

    fb_analytics_app_id = models.CharField(
        verbose_name=_('Facebook Analytics App ID'),
        max_length=25,
        null=True,
        blank=True,
        help_text=_(
            "The tracking ID to be used to view Facebook Analytics")
    )
    local_ga_tracking_code = models.CharField(
        verbose_name=_('Local GA Tracking Code'),
        max_length=255,
        null=True,
        blank=True,
        help_text=_(
            "Local GA tracking code to be used to "
            "view analytics on this site only")
    )
    global_ga_tracking_code = models.CharField(
        verbose_name=_('Global GA Tracking Code'),
        max_length=255,
        null=True,
        blank=True,
        help_text=_(
            "Global GA tracking code to be used"
            " to view analytics on more than one site globally")
    )
    show_only_translated_pages = models.BooleanField(
        default=False,
        help_text='When selecting this option, untranslated pages'
                  ' will not be visible to the front end user'
                  ' when they viewing a child language of the site')

    time = StreamField([
        ('time', blocks.TimeBlock(required=False)),
    ], null=True, blank=True, help_text='The time/s content will be rotated')

    monday_rotation = models.BooleanField(default=False, verbose_name='Monday')
    tuesday_rotation = models.BooleanField(
        default=False, verbose_name='Tuesday')
    wednesday_rotation = models.BooleanField(
        default=False, verbose_name='Wednesday')
    thursday_rotation = models.BooleanField(
        default=False, verbose_name='Thursday')
    friday_rotation = models.BooleanField(default=False, verbose_name='Friday')
    saturday_rotation = models.BooleanField(
        default=False, verbose_name='Saturday')
    sunday_rotation = models.BooleanField(default=False, verbose_name='Sunday')

    content_rotation_start_date = models.DateTimeField(
        null=True, blank=True,
        help_text='The date rotation will begin')
    content_rotation_end_date = models.DateTimeField(
        null=True, blank=True,
        help_text='The date rotation will end')

    social_media_links_on_footer_page = StreamField([
        ('social_media_site', SocialMediaLinkBlock()),
    ], null=True, blank=True)
    facebook_sharing = models.BooleanField(
        default=False, verbose_name='Facebook',
        help_text='Enable this field to allow for sharing to Facebook.')
    facebook_image = models.ForeignKey(
        'wagtailimages.Image',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+'
    )
    twitter_sharing = models.BooleanField(
        default=False, verbose_name='Twitter',
        help_text='Enable this field to allow for sharing to Twitter.')
    twitter_image = models.ForeignKey(
        'wagtailimages.Image',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+'
    )
    enable_clickable_tags = models.BooleanField(
        default=False, verbose_name='Display tags on Front-end')
    enable_tag_navigation = models.BooleanField(
        default=False,
        help_text='Enable tag navigation. When this is true, the clickable '
                  'tag functionality will be overriden'
    )
    enable_service_directory = models.BooleanField(
        default=False, verbose_name='Enable service directory'
    )
    service_directory_api_base_url = models.CharField(
        verbose_name=_('service directory base url'),
        max_length=255,
        null=True,
        blank=True,
    )
    service_directory_api_username = models.CharField(
        verbose_name=_('service directory username'),
        max_length=255,
        null=True,
        blank=True,
    )
    service_directory_api_password = models.CharField(
        verbose_name=_('service directory password'),
        max_length=255,
        null=True,
        blank=True,
    )
    google_places_api_server_key = models.CharField(
        verbose_name=_('google places server key'),
        max_length=255,
        null=True,
        blank=True,
    )

    panels = [
        ImageChooserPanel('logo'),
        MultiFieldPanel(
            [
                FieldPanel('show_only_translated_pages'),
            ],
            heading="Multi Language",
        ),
        MultiFieldPanel(
            [
                FieldPanel('fb_analytics_app_id'),
            ],
            heading="Facebook Analytics Settings",
        ),
        MultiFieldPanel(
            [
                FieldPanel('ga_tag_manager'),
                FieldPanel('global_ga_tag_manager'),
            ],
            heading="GA Tag Manager Settings",
        ),
        MultiFieldPanel(
            [
                FieldPanel('local_ga_tracking_code'),
                FieldPanel('global_ga_tracking_code'),
            ],
            heading="GA Tracking Code Settings",
        ),
        MultiFieldPanel(
            [
                FieldPanel('content_rotation_start_date'),
                FieldPanel('content_rotation_end_date'),
                FieldRowPanel([
                    FieldPanel('monday_rotation', classname='col6'),
                    FieldPanel('tuesday_rotation', classname='col6'),
                    FieldPanel('wednesday_rotation', classname='col6'),
                    FieldPanel('thursday_rotation', classname='col6'),
                    FieldPanel('friday_rotation', classname='col6'),
                    FieldPanel('saturday_rotation', classname='col6'),
                    FieldPanel('sunday_rotation', classname='col6'),
                ]),
                StreamFieldPanel('time'),
            ],
            heading="Content Rotation Settings", ),
        MultiFieldPanel(
            [
                MultiFieldPanel(
                    [
                        StreamFieldPanel('social_media_links_on_footer_page'),
                    ],
                    heading="Social Media Footer Page", ),
            ],
            heading="Social Media Footer Page Links", ),
        MultiFieldPanel(
            [
                FieldPanel('facebook_sharing'),
                ImageChooserPanel('facebook_image'),
                FieldPanel('twitter_sharing'),
                ImageChooserPanel('twitter_image'),
            ],
            heading="Social Media Article Sharing Buttons",
        ),
        MultiFieldPanel(
            [
                FieldPanel('enable_clickable_tags'),
                FieldPanel('enable_tag_navigation'),
            ],
            heading="Article Tag Settings"
        ),
        MultiFieldPanel(
            [
                FieldPanel('enable_service_directory'),
                FieldPanel('service_directory_api_base_url'),
                FieldPanel('service_directory_api_username'),
                FieldPanel('service_directory_api_password'),
                FieldPanel('google_places_api_server_key'),
            ],
            heading="Service Directory API Settings"
        )
    ]


class Timezone(models.Model):
    title = models.CharField(
        max_length=255,
        unique=True,
    )

    def __unicode__(self):
        return self.title


@register_setting
class CmsSettings(BaseSetting):
    '''
    CMS settings apply to every site running in this Django app. Wagtail
    settings are per-site, so this class applies the change to all sites when
    saving.
    '''

    timezone = models.ForeignKey(
        'core.Timezone',
        null=True,
        on_delete=models.PROTECT,
    )

    panels = [
        MultiFieldPanel(
            [
                FieldPanel('timezone'),
            ],
            heading='Core settings',
        ),
    ]

    class Meta:
        verbose_name = 'CMS settings'

    def save(self, first_site_save=True, *args, **kwargs):
        super(CmsSettings, self).save(*args, **kwargs)

        if first_site_save:
            if self.timezone is not None:
                django_timezone.activate(self.timezone.title)

            for site in Site.objects.all():
                cms_settings = CmsSettings.for_site(site)
                cms_settings.timezone = self.timezone
                cms_settings.save(first_site_save=False)


class ImageInfo(models.Model):
    image_hash = models.CharField(max_length=256, null=True)
    image = models.OneToOneField(
        'wagtailimages.Image',
        null=True,
        blank=True,
        related_name='image_info'
    )

    def save(self, *args, **kwargs):
        self.image_hash = get_image_hash(self.image)
        super(ImageInfo, self).save(*args, **kwargs)


@receiver(
    post_save, sender=Image, dispatch_uid="create_image_info")
def create_image_info(sender, instance, **kwargs):
    image_info, created = ImageInfo.objects.get_or_create(image=instance)
    # ensure that image info is updated, in the event that an
    # image is changed e.g. file is changed, prompting change in hash
    if not created:
        image_info.save()


class ImportableMixin(object):
    @classmethod
    def create_page(self, content, class_, record_keeper=None, logger=None):
        '''
        Robust as possible

        Attempts to create the page
        If any of the functions used to attach content to the page
        fail, keep going, keep a record of those errors in a context dict
        return the page and the context dict in a tuple
        '''
        fields, nested_fields = separate_fields(content)

        foreign_id = content.pop('id')

        # remove unwanted fields
        if 'latest_revision_created_at' in content:
            content.pop('latest_revision_created_at')

        page = class_(**fields)

        # create functions to attach attributes
        function_args_mapping = (
            # add_section_time
            (add_json_dump, ("time", nested_fields, page)),
            # add_tags
            (add_list_of_things, ("tags", nested_fields, page)),
            # add_metadata_tags
            (add_list_of_things, ("metadata_tags", nested_fields, page)),

            # attach_image
            (attach_image, ("image", nested_fields, page, record_keeper)),
            # attach_social_media_image
            (attach_image, ("social_media_image", nested_fields,
                            page, record_keeper)),
            # attach_banner_image
            (attach_image, ("banner", nested_fields, page, record_keeper)),
        )

        for mapping in function_args_mapping:
            function = mapping[0]
            _args = mapping[1]
            try:
                function(*_args)
            except Exception as e:
                if logger:
                    logger.log(
                        ERROR,
                        "Failed to create page content",
                        {
                            "foreign_page_id": foreign_id,
                            "exception": e,
                            "function": function.__name__,
                        })

        # Handle content in nested_fields
        body = add_stream_fields(nested_fields, page)
        # body has not been added as it contains reference to pages
        if body:
            record_keeper.article_bodies[foreign_id] = body

        # Handle relationships in nested_fields
        if record_keeper:
            record_relation_functions = [
                record_keeper.record_nav_tags,
                record_keeper.record_recommended_articles,
                record_keeper.record_reaction_questions,
                record_keeper.record_related_sections,
                record_keeper.record_section_tags,
                record_keeper.record_banner_page_link,
            ]

            for function in record_relation_functions:
                try:
                    function(nested_fields, foreign_id)
                except Exception as e:
                    if logger:
                        logger.log(
                            ERROR,
                            "Failed to record content",
                            {
                                "foreign_page_id": foreign_id,
                                "exception": e,
                                "function": function.__name__,
                            })

        return page


class PreventDeleteMixin(object):
    hide_delete_button = True


class MoloMedia(AbstractMedia):
    youtube_link = models.CharField(max_length=512, null=True, blank=True)
    feature_in_homepage = models.BooleanField(default=False)

    admin_form_fields = (
        'title',
        'file',
        'collection',
        'duration',
        'width',
        'height',
        'thumbnail',
        'tags',
        'youtube_link',
        'feature_in_homepage',
    )


class MoloMediaBlock(AbstractMediaChooserBlock):
    def render_basic(self, value, context):
        if not value:
            return ''

        if value.type == 'video':
            player_code = '''
            <div>
                <video width="320" height="240" controls>
                    <source src="{0}" type="video/mp4">
                    Your browser does not support the video tag.
                </video>
            </div>
            '''
        else:
            player_code = '''
            <div>
                <audio controls>
                    <source src="{0}" type="audio/mpeg">
                    Your browser does not support the audio element.
                </audio>
            </div>
            '''
        return format_html(player_code, value.file.url)


class CommentedPageMixin(object):
    def get_effective_commenting_settings(self):
        # return commenting settings for the homepage
        if self.commenting_state:
            return {
                'state': self.commenting_state,
                'open_time': self.commenting_open_time,
                'close_time': self.commenting_close_time
            }
        # use the commenting settings for the parent page

        parent_page = Page.objects.all().ancestor_of(self).last()
        if parent_page:
            parent = parent_page.specific
            return parent.get_effective_commenting_settings()
        # add a default in case nothing is set. We should never get here
        return {
            'state': constants.COMMENTING_DISABLED,
            'open_time': None,
            'close_time': None
        }


class PageTranslation(models.Model):
    page = models.ForeignKey('wagtailcore.Page', related_name='translations')
    translated_page = models.OneToOneField(
        'wagtailcore.Page', related_name='source_page')


class LanguageRelation(models.Model):
    page = models.ForeignKey('wagtailcore.Page', related_name='languages')
    language = models.ForeignKey('core.SiteLanguage', related_name='+')


class TranslatablePageMixinNotRoutable(object):
    def get_translation_for_cache_key(self, locale, site, is_live):
        return "get_translation_for_{}_{}_{}_{}_{}".format(
            self.pk, locale, site.pk, is_live,
            self.latest_revision_created_at.isoformat())

    def get_translation_for(self, locale, site, is_live=True):
        cache_key = self.get_translation_for_cache_key(locale, site, is_live)
        trans_pk = cache.get(cache_key)

        # TODO: consider pickling page object. Be careful about page size in
        # memory
        if trans_pk:
            return Page.objects.get(pk=trans_pk).specific

        language_setting = Languages.for_site(site)
        language = language_setting.languages.filter(
            locale=locale).first()

        if not language:
            return None

        main_language_page = self.get_main_language_page()
        if language.is_main_language and not self == main_language_page:
            cache.set(cache_key, main_language_page.pk, None)
            return main_language_page

        translated = None
        qs = self.specific.translations.all()
        if is_live is not None:
            qs = qs.filter(translated_page__live=True)

        for t in qs:
            if t.translated_page.languages.filter(
                    language__locale=locale).exists():
                translated = t.translated_page.languages.filter(
                    language__locale=locale).first().page.specific
                break
        if translated:
            cache.set(cache_key, translated.pk, None)
        return translated

    def get_main_language_page(self):
        if hasattr(self.specific, 'source_page') and self.specific.source_page:
            return self.specific.source_page.page.specific
        return self.specific

    def get_site(self):
        # TODO: this will need to change for one content repo work
        return self.get_ancestors().filter(
            depth=2).first().sites_rooted_here.all().first() or None

    def save(self, *args, **kwargs):
        response = super(
            TranslatablePageMixinNotRoutable, self).save(*args, **kwargs)
        languages = Languages.for_site(self.get_site()).languages
        if (languages.filter(
                is_main_language=True).exists() and
                not self.languages.exists()):
            LanguageRelation.objects.create(
                page=self,
                language=languages.filter(
                    is_main_language=True).first())
        return response

    def move(self, target, pos=None):
        super(TranslatablePageMixinNotRoutable, self).move(target, pos)

        if hasattr(self, 'translations'):
            for p in self.translations.all():
                p.translated_page.move(target, pos='last-child')

    def copy_language(self, current_site, destination_site):
        language = self.languages.all().first()
        if language:
            if not hasattr(destination_site, 'languages') or \
                not destination_site.languages.languages.filter(
                    locale=language.language.locale).exists():
                new_lang = SiteLanguageRelation.objects.create(
                    language_setting=Languages.for_site(destination_site),
                    locale=language.language.locale,
                    is_active=False)
            else:
                new_lang = destination_site.languages.languages.filter(
                    locale=language.language.locale).first()
            return new_lang

    def copy(self, *args, **kwargs):
        current_site = self.get_site()
        destination_site = kwargs['to'].get_site()
        if current_site is not destination_site:
            new_lang = self.copy_language(current_site, destination_site)
            page_copy = super(TranslatablePageMixinNotRoutable, self).copy(
                *args, **kwargs)

            if new_lang:
                if LanguageRelation.objects.filter(page=page_copy).exists():
                    new_l_rel = LanguageRelation.objects.get(page=page_copy)
                    new_l_rel.language = new_lang
                    new_l_rel.save()
                else:
                    new_l_rel = LanguageRelation.objects.create(
                        page=page_copy, language=new_lang)

            old_parent = self.get_main_language_page()
            if old_parent:
                new_translation_parent = \
                    page_copy.get_parent().get_children().filter(
                        slug=old_parent.slug).first()
                if new_translation_parent:
                    PageTranslation.objects.create(
                        page=new_translation_parent,
                        translated_page=page_copy)
            return page_copy
        else:
            return super(TranslatablePageMixinNotRoutable, self).copy(
                *args, **kwargs)

    @route(r'^noredirect/$')
    def noredirect(self, request):
        return Page.serve(self, request)

    @route(r'^$')
    def main(self, request):
        return TemplateResponse(
            request, self.get_template(request),
            self.get_context(request))

    def get_sitemap_urls(self):
        return [
            {
                'location': self.full_url,
                'lastmod': self.latest_revision_created_at
            }
        ]

    def serve(self, request, *args, **kwargs):
        locale_code = get_locale_code(get_language_from_request(request))
        parent = self.get_main_language_page()
        translation = parent.specific.get_translation_for(
            locale_code, request.site)
        language_rel = self.languages.all().first()

        main_lang = Languages.for_site(request.site).languages.filter(
            is_main_language=True).first()
        if main_lang.locale == locale_code:
            translation = parent

        if translation and language_rel.language.locale != locale_code:
            if request.GET.urlencode():
                return redirect("{}?{}".format(translation.url,
                                               request.GET.urlencode()))
            else:
                return redirect(translation.url)

        return super(TranslatablePageMixinNotRoutable, self).serve(
            request, *args, **kwargs)


def clear_translation_cache(sender, instance, **kwargs):
    if isinstance(instance, TranslatablePageMixin):
        site = instance.get_site()
        for lang in Languages.for_site(site).languages.all():
            cache.delete(instance.get_translation_for_cache_key(
                lang.locale, site, True))
            cache.delete(instance.get_translation_for_cache_key(
                lang.locale, site, False))

            # clear cache for main language page too
            parent = instance.get_main_language_page()
            cache.delete(parent.get_translation_for_cache_key(
                lang.locale, site, True))
            cache.delete(parent.get_translation_for_cache_key(
                lang.locale, site, False))


page_unpublished.connect(clear_translation_cache)


class TranslatablePageMixin(
        TranslatablePageMixinNotRoutable, RoutablePageMixin):
    pass


class TagIndexPage(MoloPage, PreventDeleteMixin):
    parent_page_types = []
    subpage_types = ['Tag']

    def copy(self, *args, **kwargs):
        site = kwargs['to'].get_site()
        main = site.root_page
        TagIndexPage.objects.child_of(main).delete()
        super(TagIndexPage, self).copy(*args, **kwargs)


class ReactionQuestionIndexPage(MoloPage, PreventDeleteMixin):
    parent_page_types = []
    subpage_types = ['ReactionQuestion']

    def copy(self, *args, **kwargs):
        site = kwargs['to'].get_site()
        main = site.root_page
        ReactionQuestionIndexPage.objects.child_of(main).delete()
        super(ReactionQuestionIndexPage, self).copy(*args, **kwargs)


class ReactionQuestion(TranslatablePageMixin, MoloPage):
    parent_page_types = ['core.ReactionQuestionIndexPage']
    subpage_types = ['ReactionQuestionChoice']

    def has_user_submitted_reaction_response(
            self, request, reaction_id, article_id):
        if 'reaction_response_submissions' not in request.session:
            request.session['reaction_response_submissions'] = []
        if article_id in request.session['reaction_response_submissions']:
            return True
        return False


class ReactionQuestionChoice(TranslatablePageMixinNotRoutable,
                             PageEffectiveImageMixin, MoloPage):
    parent_page_types = ['core.ReactionQuestion']
    subpage_types = []

    image = models.ForeignKey(
        'wagtailimages.Image',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+'
    )

    success_message = models.CharField(blank=True, null=True, max_length=1000)
    success_image = models.ForeignKey(
        'wagtailimages.Image',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+'
    )


ReactionQuestionChoice.content_panels = [
    FieldPanel('title', classname='full title'),
    ImageChooserPanel('image'),
    FieldPanel('success_message', classname='full title'),
    ImageChooserPanel('success_image'),
]


class ReactionQuestionResponse(models.Model):
    user = models.ForeignKey('auth.User', blank=True, null=True)
    article = models.ForeignKey('core.ArticlePage')
    choice = models.ForeignKey(
        'core.ReactionQuestionChoice', blank=True, null=True)
    question = models.ForeignKey('core.ReactionQuestion')
    created_at = models.DateTimeField(auto_now_add=True)

    def set_response_as_submitted_for_session(self, request, article):
        if 'reaction_response_submissions' not in request.session:
            request.session['reaction_response_submissions'] = []
        request.session['reaction_response_submissions'].append(article.id)
        request.session.modified = True

    class Meta:
        permissions = (
            ("can_view_response", "Can view Response"),
        )


class Tag(TranslatablePageMixin, MoloPage, ImportableMixin):
    parent_page_types = ['core.TagIndexPage']
    subpage_types = []

    feature_in_homepage = models.BooleanField(default=False)

    api_fields = [
        "id", "title", "feature_in_homepage", "go_live_at",
        "expire_at", "expired"
    ]


Tag.promote_panels = [
    FieldPanel('feature_in_homepage'),
    MultiFieldPanel(
        Page.promote_panels,
        "Common page configuration", "collapsible collapsed")
]


class BannerIndexPage(MoloPage, PreventDeleteMixin, ImportableMixin):
    parent_page_types = []
    subpage_types = ['BannerPage']

    def copy(self, *args, **kwargs):
        site = kwargs['to'].get_site()
        main = site.root_page
        BannerIndexPage.objects.child_of(main).delete()
        super(BannerIndexPage, self).copy(*args, **kwargs)


class BannerPage(ImportableMixin, TranslatablePageMixin, MoloPage):
    parent_page_types = ['core.BannerIndexPage']
    subpage_types = []

    subtitle = models.TextField(null=True, blank=True)
    banner = models.ForeignKey(
        'wagtailimages.Image',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+'
    )
    banner_link_page = models.ForeignKey(
        'wagtailcore.Page',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        help_text=_('Optional page to which the banner will link to')
    )
    external_link = models.TextField(null=True, blank=True,
                                     help_text='External link which a banner'
                                     ' will link to. '
                                     'eg https://www.google.co.za/')
    api_fields = [
        "title", "subtitle", "banner", "banner_link_page", "external_link"]

    def get_effective_banner(self):
        if self.banner:
            return self.banner
        page = self.get_main_language_page()
        if page.specific.banner:
            return page.specific.get_effective_banner()
        return ''


BannerPage.content_panels = [
    FieldPanel('title', classname='full title'),
    FieldPanel('subtitle'),
    ImageChooserPanel('banner'),
    PageChooserPanel('banner_link_page'),
    FieldPanel('external_link')
]

# Signal for allowing plugins to create indexes
index_pages_after_copy = Signal(providing_args=["instance"])


class Main(CommentedPageMixin, MoloPage):
    subpage_types = []

    def bannerpages(self):
        index_page = BannerIndexPage.objects.child_of(self).live().first()
        return BannerPage.objects.child_of(index_page).filter(
            languages__language__is_main_language=True).specific()

    def sections(self):
        index_page = SectionIndexPage.objects.child_of(self).live().first()
        return SectionPage.objects.child_of(index_page).filter(
            languages__language__is_main_language=True).specific()

    def latest_articles(self):
        return ArticlePage.objects.descendant_of(self).filter(
            featured_in_latest=True,
            languages__language__is_main_language=True).exclude(
                feature_as_topic_of_the_day=True,
                demote_date__gt=django_timezone.now()).order_by(
                    '-featured_in_latest_start_date',
                    '-promote_date', '-latest_revision_created_at').specific()

    def topic_of_the_day(self):
        return ArticlePage.objects.descendant_of(self).filter(
            feature_as_topic_of_the_day=True,
            languages__language__is_main_language=True,
            promote_date__lte=django_timezone.now(),
            demote_date__gte=django_timezone.now()).order_by(
            '-promote_date').specific()

    def footers(self):
        return FooterPage.objects.descendant_of(self).filter(
            languages__language__is_main_language=True).specific()

    def save(self, *args, **kwargs):
        super(Main, self).save(*args, **kwargs)
        if not self.get_descendants().exists():
            banner_index = BannerIndexPage(
                title='Banners', slug=('banners-%s' % (
                    generate_slug(self.title), )))
            self.add_child(instance=banner_index)
            banner_index.save_revision().publish()
            section_index = SectionIndexPage(
                title='Sections', slug=('sections-%s' % (
                    generate_slug(self.title), )))
            self.add_child(instance=section_index)
            section_index.save_revision().publish()
            footer_index = FooterIndexPage(
                title='Footers', slug=('footers-%s' % (
                    generate_slug(self.title), )))
            self.add_child(instance=footer_index)
            footer_index.save_revision().publish()
            tag_index = TagIndexPage(
                title='Tags', slug=('tags-%s' % (
                    generate_slug(self.title), )))
            self.add_child(instance=tag_index)
            tag_index.save_revision().publish()
            reaction_question_index = ReactionQuestionIndexPage(
                title='Reaction Questions', slug=('reaction-questions-%s' % (
                    generate_slug(self.title), )))
            self.add_child(instance=reaction_question_index)
            reaction_question_index.save_revision().publish()
            index_pages_after_copy.send(sender=self.__class__, instance=self)


@receiver(
    post_save, sender=Main, dispatch_uid="create_site")
def create_site(sender, instance, **kwargs):
    default_site = not(Site.objects.all().exists())

    if not hasattr(settings, 'DEFAULT_SITE_PORT'):
        port = 80
    else:
        port = settings.DEFAULT_SITE_PORT
    # create site
    if not instance.sites_rooted_here.exists():
        site = Site(
            hostname=generate_slug(instance.title) + '.localhost',
            port=port, root_page=instance,
            is_default_site=default_site)
        site.save()


class LanguagePage(CommentedPageMixin, MoloPage):
    code = models.CharField(
        max_length=255,
        help_text=_('The language code as specified in iso639-2'))

    parent_page_types = []
    subpage_types = []

    commenting_state = models.CharField(
        max_length=1,
        choices=constants.COMMENTING_STATE_CHOICES,
        blank=True,
        null=True)
    commenting_open_time = models.DateTimeField(null=True, blank=True)
    commenting_close_time = models.DateTimeField(null=True, blank=True)

    class Meta:
        verbose_name = _('Language')


LanguagePage.content_panels = [
    FieldPanel('title', classname='full title'),
    FieldPanel('code'),

    MultiFieldPanel(
        [
            FieldPanel('commenting_state'),
            FieldPanel('commenting_open_time'),
            FieldPanel('commenting_close_time'),
        ],
        heading="Commenting Settings", )
]


@register_setting
class Languages(BaseSetting, ClusterableModel):

    panels = [
        InlinePanel('languages', label="Site Languages"),
    ]


class SiteLanguage(models.Model):
    locale = models.CharField(
        verbose_name=_('language name'),
        choices=settings.LANGUAGES,
        max_length=255,
        blank=False,
        help_text=_("Site language")
    )
    is_main_language = models.BooleanField(
        default=False,
        editable=False,
        verbose_name=_('main Language'),
    )
    is_active = models.BooleanField(
        default=True,
        verbose_name=_('active Language'),
    )

    def save(self, *args, **kwargs):
        if Languages.for_site(self.language_setting.site).languages.filter(
                is_main_language=True).exists():
            return super(SiteLanguage, self).save(*args, **kwargs)

        self.is_main_language = True
        return super(SiteLanguage, self).save(*args, **kwargs)

    def __str__(self):  # pragma: no cover
        return "%s" % (self.get_locale_display(),)

    class Meta:
        verbose_name = _('Language')

    panels = [
        FieldPanel('locale'),
        FieldPanel('is_active'),
        ReadOnlyPanel('is_main_language')
    ]

    api_fields = ["locale", "is_main_language", "is_active"]


class SiteLanguageRelation(Orderable, SiteLanguage):
    language_setting = ParentalKey(Languages, related_name='languages')


class SectionIndexPage(CommentedPageMixin, MoloPage, PreventDeleteMixin):
    parent_page_types = []
    subpage_types = ['SectionPage']

    commenting_state = models.CharField(
        max_length=1,
        choices=constants.COMMENTING_STATE_CHOICES,
        blank=True,
        null=True,
        default=constants.COMMENTING_OPEN)
    commenting_open_time = models.DateTimeField(null=True, blank=True)
    commenting_close_time = models.DateTimeField(null=True, blank=True)

    def celery_copy(self, *args, **kwargs):
        SectionIndexPage.objects.child_of(kwargs['to']).delete()
        return super(SectionIndexPage, self).copy(*args, **kwargs)

    def copy(self, *args, **kwargs):
        from molo.core.tasks import copy_sections_index

        via_celery = kwargs.get('via_celery')

        if via_celery:
            del kwargs['via_celery']
            return self.celery_copy(*args, **kwargs)

        user = kwargs.get('user')
        user_pk = user.pk if user else None
        to_pk = kwargs['to'].pk
        copy_revisions = kwargs.get('copy_revisions')
        recursive = kwargs.get('recursive')
        keep_live = kwargs.get('keep_live')

        # workaround for celery tasks
        # https://github.com/celery/django-celery/issues/201
        if hasattr(settings, 'CELERY_ALWAYS_EAGER') and \
                settings.CELERY_ALWAYS_EAGER and settings.DEBUG:
            copy_sections_index(
                self.pk, user_pk, to_pk, copy_revisions, recursive, keep_live)
        else:
            copy_sections_index.delay(
                self.pk, user_pk, to_pk, copy_revisions, recursive, keep_live)


SectionIndexPage.content_panels = [
    FieldPanel('title', classname='full title'),
    MultiFieldPanel(
        [
            FieldPanel('commenting_state'),
            FieldPanel('commenting_open_time'),
            FieldPanel('commenting_close_time'),
        ],
        heading="Commenting Settings", )
]


class SectionPage(ImportableMixin, CommentedPageMixin,
                  TranslatablePageMixin, MoloPage):
    description = models.TextField(null=True, blank=True)
    uuid = models.CharField(max_length=32, blank=True, null=True)
    image = models.ForeignKey(
        'wagtailimages.Image',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+'
    )
    parent_page_types = ['core.SectionIndexPage', 'core.SectionPage']
    subpage_types = ['core.ArticlePage', 'core.SectionPage']
    search_fields = Page.search_fields + [
        index.SearchField('description'),
    ]
    extra_style_hints = models.TextField(
        default='',
        null=True, blank=True,
        help_text=_(
            "Styling options that can be applied to this section "
            "and all its descendants"))

    commenting_state = models.CharField(
        max_length=1,
        choices=constants.COMMENTING_STATE_CHOICES,
        blank=True,
        null=True)
    commenting_open_time = models.DateTimeField(null=True, blank=True)
    commenting_close_time = models.DateTimeField(null=True, blank=True)

    time = StreamField([
        ('time', blocks.TimeBlock(required=False)),
    ], null=True, blank=True, help_text='The time/s content will be rotated')

    monday_rotation = models.BooleanField(default=False, verbose_name='Monday')
    tuesday_rotation = models.BooleanField(
        default=False, verbose_name='Tuesday')
    wednesday_rotation = models.BooleanField(
        default=False, verbose_name='Wednesday')
    thursday_rotation = models.BooleanField(
        default=False, verbose_name='Thursday')
    friday_rotation = models.BooleanField(default=False, verbose_name='Friday')
    saturday_rotation = models.BooleanField(
        default=False, verbose_name='Saturday')
    sunday_rotation = models.BooleanField(default=False, verbose_name='Sunday')

    content_rotation_start_date = models.DateTimeField(
        null=True, blank=True,
        help_text='The date rotation will begin')
    content_rotation_end_date = models.DateTimeField(
        null=True, blank=True,
        help_text='The date rotation will end')

    enable_next_section = (
        models.BooleanField(
            default=False,
            verbose_name='Activate up next section underneath articles',
            help_text=("Activate up next section underneath articles in this "
                       "section will appear with the heading and subheading of"
                       " that article. The text will say 'next' in order to "
                       "make the user feel like it's fresh content.")))
    enable_recommended_section = (
        models.BooleanField(
            default=False,
            verbose_name='Activate recommended section underneath articles',
            help_text=("Underneath the area for 'next articles' recommended "
                       "articles will appear, with the image + heading + "
                       "subheading")))

    api_fields = [
        "title", "live", "description", "image", "extra_style_hints",
        "commenting_state", "commenting_open_time",
        "commenting_close_time", "time", "monday_rotation",
        "tuesday_rotation", "wednesday_rotation", "thursday_rotation",
        "friday_rotation", "saturday_rotation", "sunday_rotation",
        "content_rotation_start_date", "content_rotation_end_date",
        "section_tags", "enable_next_section", "enable_recommended_section",
        "go_live_at", "expire_at", "expired"
    ]

    @classmethod
    def get_api_fields(cls):
        return cls.api_fields

    def articles(self):
        main_language_page = self.get_main_language_page()
        return list(chain(
            ArticlePage.objects.child_of(main_language_page).filter(
                languages__language__is_main_language=True),
            ArticlePage.objects.filter(
                related_sections__section__slug=main_language_page.slug)
        )
        )

    def sections(self):
        main_language_page = self.get_main_language_page()
        return SectionPage.objects.child_of(main_language_page).filter(
            languages__language__is_main_language=True)

    def get_site(self):
        main = self.get_ancestors().filter(
            depth=2).first()
        return main.sites_rooted_here.all().first()

    def get_effective_extra_style_hints(self):
        cache_key = "effective_extra_style_hints_{}_{}".format(
            self.pk, self.latest_revision_created_at.isoformat())
        style = cache.get(cache_key)

        if style is not None:
            return style

        if self.extra_style_hints:
            return self.extra_style_hints

        # The extra css is inherited from the parent SectionPage.
        # This will either return the current value or a value
        # from its parents.
        main_lang = Languages.for_site(self.get_site()).languages.filter(
            is_main_language=True).first()

        language_rel = self.languages.all().first()

        if not main_lang or not language_rel:
            return ''

        if language_rel and main_lang.pk == language_rel.language.pk:
            parent_section = SectionPage.objects.all().ancestor_of(self).last()
            if parent_section:
                style = parent_section.get_effective_extra_style_hints()
                cache.set(cache_key, style, 300)
                return style
            return ''
        else:
            page = self.get_main_language_page()
            style = page.specific.get_effective_extra_style_hints()
            cache.set(cache_key, style, 300)
            return style

    def get_effective_image(self):
        if self.image:
            return self.image

        main_lang = Languages.for_site(self.get_site()).languages.filter(
            is_main_language=True).first()
        language_rel = self.languages.all().first()
        if language_rel and main_lang.pk == language_rel.language.pk:
            parent_section = SectionPage.objects.all().ancestor_of(self).last()
            if parent_section:
                return parent_section.get_effective_image()
            return ''
        else:
            page = self.get_main_language_page()
            return page.specific.get_effective_image()

    def get_parent_section(self):
        return SectionPage.objects.all().ancestor_of(self).last()

    def featured_in_homepage_articles(self):
        main_language_page = self.get_main_language_page()
        return ArticlePage.objects.child_of(main_language_page).filter(
            languages__language__is_main_language=True,
            featured_in_homepage=True).order_by(
                '-latest_revision_created_at').specific()

    def get_context(self, request):
        context = super(SectionPage, self).get_context(request)

        try:
            p = int(request.GET.get('p', 1))
        except ValueError:
            p = 1

        context['p'] = p
        return context

    class Meta:
        verbose_name = _('Section')


SectionPage.content_panels = [
    FieldPanel('title', classname='full title'),
    FieldPanel('description'),
    ImageChooserPanel('image'),
    MultiFieldPanel(
        [
            FieldPanel('commenting_state'),
            FieldPanel('commenting_open_time'),
            FieldPanel('commenting_close_time'),
        ],
        heading="Commenting Settings", ),
    InlinePanel('section_tags', label="Tags for Navigation"),
]

SectionPage.settings_panels = [
    MultiFieldPanel(
        Page.settings_panels, "Scheduled publishing", "publishing"),
    MultiFieldPanel(
        [
            FieldPanel('content_rotation_start_date'),
            FieldPanel('content_rotation_end_date'),
            FieldRowPanel([
                FieldPanel('monday_rotation', classname='col6'),
                FieldPanel('tuesday_rotation', classname='col6'),
                FieldPanel('wednesday_rotation', classname='col6'),
                FieldPanel('thursday_rotation', classname='col6'),
                FieldPanel('friday_rotation', classname='col6'),
                FieldPanel('saturday_rotation', classname='col6'),
                FieldPanel('sunday_rotation', classname='col6'),
            ]),
            StreamFieldPanel('time'),
        ],
        heading="Content Rotation Settings", ),
    MultiFieldPanel(
        [FieldRowPanel(
            [FieldPanel('extra_style_hints')], classname="label-above")],
        "Meta"),
    MultiFieldPanel(
        [
            FieldPanel('enable_next_section'),
            FieldPanel('enable_recommended_section')
        ],
        heading="Recommended Settings", )
]


class ArticlePageTag(TaggedItemBase):
    content_object = ParentalKey(
        'core.ArticlePage', related_name='tagged_items')


class ArticlePageMetaDataTag(TaggedItemBase):
    content_object = ParentalKey(
        'core.ArticlePage', related_name='metadata_tagged_items')


class ArticlePage(ImportableMixin, CommentedPageMixin,
                  TranslatablePageMixin, PageEffectiveImageMixin, MoloPage):
    parent_page_types = ['core.SectionPage']

    subtitle = models.TextField(null=True, blank=True)
    uuid = models.CharField(max_length=32, blank=True, null=True)
    featured_in_latest = models.BooleanField(
        default=False,
        help_text=_("Article to be featured in the Latest module"))
    featured_in_latest_start_date = models.DateTimeField(null=True, blank=True)
    featured_in_latest_end_date = models.DateTimeField(null=True, blank=True)
    featured_in_section = models.BooleanField(
        default=False,
        help_text=_("Article to be featured in the Section module"))
    featured_in_section_start_date = models.DateTimeField(
        null=True, blank=True)
    featured_in_section_end_date = models.DateTimeField(null=True, blank=True)
    featured_in_homepage = models.BooleanField(
        default=False,
        help_text=_(
            "Article to be featured in the Homepage "
            "within the Section module"))
    featured_in_homepage_start_date = models.DateTimeField(
        null=True, blank=True)
    featured_in_homepage_end_date = models.DateTimeField(null=True, blank=True)
    image = models.ForeignKey(
        'wagtailimages.Image',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+'
    )
    social_media_title = models.TextField(null=True, blank=True,
                                          verbose_name="title")
    social_media_description = models.TextField(null=True, blank=True,
                                                verbose_name="description")
    social_media_image = models.ForeignKey(
        'wagtailimages.Image',
        null=True,
        blank=True,
        verbose_name="Image",
        on_delete=models.SET_NULL,
        related_name='+'
    )
    body = StreamField([
        ('heading', blocks.CharBlock(classname="full title")),
        ('paragraph', MarkDownBlock()),
        ('image', ImageChooserBlock()),
        ('list', blocks.ListBlock(blocks.CharBlock(label="Item"))),
        ('numbered_list', blocks.ListBlock(blocks.CharBlock(label="Item"))),
        ('page', blocks.PageChooserBlock()),
        ('media', MoloMediaBlock(icon='media'),)
    ], null=True, blank=True)

    tags = ClusterTaggableManager(through=ArticlePageTag, blank=True)
    metadata_tags = ClusterTaggableManager(
        through=ArticlePageMetaDataTag,
        blank=True, related_name="metadata_tags",
        help_text=_(
            'A comma-separated list of tags. '
            'This is not visible to the user.'))

    subpage_types = []
    search_fields = Page.search_fields + [
        index.SearchField('subtitle'),
        index.SearchField('body'),
        index.RelatedFields('tags', [
            index.SearchField('name', partial_match=True, boost=2),
        ]),
    ]

    commenting_state = models.CharField(
        max_length=1,
        choices=constants.COMMENTING_STATE_CHOICES,
        blank=True,
        null=True)
    commenting_open_time = models.DateTimeField(null=True, blank=True)
    commenting_close_time = models.DateTimeField(null=True, blank=True)

    feature_as_topic_of_the_day = models.BooleanField(
        default=False,
        help_text=_('Article to be featured as the Topic of the Day'))
    promote_date = models.DateTimeField(blank=True, null=True)
    demote_date = models.DateTimeField(blank=True, null=True)

    featured_latest_promote_panels = [
        FieldPanel('featured_in_latest_start_date'),
        FieldPanel('featured_in_latest_end_date'),
    ]
    featured_section_promote_panels = [
        FieldPanel('featured_in_section_start_date'),
        FieldPanel('featured_in_section_end_date'),

    ]
    featured_homepage_promote_panels = [
        FieldPanel('featured_in_homepage_start_date'),
        FieldPanel('featured_in_homepage_end_date'),
    ]

    topic_of_the_day_panels = [
        FieldPanel('feature_as_topic_of_the_day'),
        FieldPanel('promote_date'),
        FieldPanel('demote_date'),
    ]

    metedata_promote_panels = [
        FieldPanel('metadata_tags'),
    ]

    base_form_class = ArticlePageForm

    def move(self, *args, **kwargs):
        current_site = self.get_site()
        destination_site = args[0].get_site()

        if not (current_site is destination_site):
            for language in self.languages.all():
                if not destination_site.languages.languages.filter(
                        locale=language.language.locale).exists():
                    new_lang = SiteLanguageRelation.objects.create(
                        language_setting=Languages.for_site(destination_site),
                        locale=language.language.locale,
                        is_active=False)
                    LanguageRelation.objects.create(
                        page=self, language=new_lang)
        super(ArticlePage, self).move(*args, **kwargs)

    def get_absolute_url(self):  # pragma: no cover
        return self.url

    def get_parent_section(self):
        return SectionPage.objects.all().ancestor_of(self).last()

    def allow_commenting(self):
        commenting_settings = self.get_effective_commenting_settings()
        if (commenting_settings['state'] != constants.COMMENTING_OPEN):
            now = django_timezone.now()
            if (commenting_settings['state'] ==
                    constants.COMMENTING_TIMESTAMPED):
                # Allow commenting over the given time period
                open_time = commenting_settings['open_time']
                close_time = commenting_settings['close_time']
                return open_time < now < close_time
            if (commenting_settings['state'] == constants.COMMENTING_CLOSED or
                    commenting_settings['state'] ==
                    constants.COMMENTING_DISABLED):
                # Allow automated reopening of commenting at a specified time
                reopen_time = commenting_settings['open_time']
                if (reopen_time):
                    if reopen_time < now:
                        self.commenting_state = constants.COMMENTING_OPEN
                        self.save()
                        return True
            return False
        return True

    def is_current_topic_of_the_day(self):
        if self.feature_as_topic_of_the_day:
            now = django_timezone.now()
            return self.promote_date <= now <= self.demote_date
        return False

    def is_commenting_enabled(self):
        commenting_settings = self.get_effective_commenting_settings()
        if (commenting_settings['state'] == constants.COMMENTING_DISABLED or
                commenting_settings['state'] is None):
            return False
        return True

    def tags_list(self):
        return self.tags.names()

    class Meta:
        verbose_name = _('Article')
        ordering = ('-latest_revision_created_at',)

    api_fields = [
        "title", "subtitle", "body", "tags", "commenting_state",
        "commenting_open_time", "commenting_close_time", "social_media_title",
        "social_media_description", "social_media_image", "related_sections",
        "featured_in_latest", "featured_in_latest_start_date",
        "featured_in_latest_end_date", "featured_in_section",
        "featured_in_section_start_date", "featured_in_section_end_date",
        "featured_in_homepage", "featured_in_homepage_start_date",
        "featured_in_homepage_end_date", "feature_as_topic_of_the_day",
        "promote_date", "demote_date", "metadata_tags",
        "latest_revision_created_at", "image",
        "social_media_image", "social_media_description",
        "social_media_title", "reaction_questions",
        "nav_tags", "recommended_articles", "related_sections",
        "go_live_at", "expire_at", "expired"
    ]

    @classmethod
    def get_api_fields(cls):
        return cls.api_fields


ArticlePage.content_panels = [
    FieldPanel('title', classname='full title'),
    FieldPanel('subtitle'),
    ImageChooserPanel('image'),
    StreamFieldPanel('body'),
    FieldPanel('tags'),
    MultiFieldPanel(
        [
            FieldPanel('commenting_state'),
            FieldPanel('commenting_open_time'),
            FieldPanel('commenting_close_time'),
        ],
        heading="Commenting Settings", ),
    MultiFieldPanel(
        [
            FieldPanel('social_media_title'),
            FieldPanel('social_media_description'),
            ImageChooserPanel('social_media_image'),
        ],
        heading="Social Media", ),
    InlinePanel('nav_tags', label="Tags for Navigation"),
    InlinePanel('reaction_questions', label="Reaction Questions"),
    InlinePanel('recommended_articles', label="Recommended articles"),
    InlinePanel('related_sections', label="Related Sections"),
]

ArticlePage.promote_panels = [
    MultiFieldPanel(
        ArticlePage.featured_latest_promote_panels, "Featuring in Latest"),
    MultiFieldPanel(
        ArticlePage.featured_section_promote_panels, "Featuring in Section"),
    MultiFieldPanel(
        ArticlePage.featured_homepage_promote_panels, "Featuring in Homepage"),
    MultiFieldPanel(ArticlePage.topic_of_the_day_panels, "Topic of the Day"),
    MultiFieldPanel(ArticlePage.metedata_promote_panels, "Metadata"),
    MultiFieldPanel(
        Page.promote_panels,
        "Common page configuration", "collapsible collapsed")]


@receiver(
    pre_save, sender=ArticlePage, dispatch_uid="demote_featured_articles")
def demote_featured_articles(sender, instance, **kwargs):
    if instance.featured_in_latest_end_date is None and \
        instance.featured_in_latest_start_date is None and \
            instance.featured_in_latest is True:
        instance.featured_in_latest = False
    if instance.featured_in_homepage_end_date is None and \
        instance.featured_in_homepage_start_date is None and \
            instance.featured_in_homepage is True:
        instance.featured_in_homepage = False
    if instance.featured_in_section_end_date is None and \
        instance.featured_in_section_start_date is None and \
            instance.featured_in_section is True:
        instance.featured_in_section = False


class ArticlePageLanguageManager(PageManager):
    def get_queryset(self):
        return super(ArticlePageLanguageManager, self).get_queryset().filter(
            languages__language__is_main_language=True
        )


class ArticlePageLanguageProxy(ArticlePage):
    class Meta:
        proxy = True
        verbose_name = _('Article View')
        verbose_name_plural = _('Article View')

    objects = ArticlePageLanguageManager()

    @classmethod
    def get_indexed_objects(cls):
        '''
        Wagtail's ElasticSearch indexing adds all instances of
        a given model to the index which is causing duplicate
        articles in the search result.
        This get_indexed_objects method will exclude
        ArticlePageLanguageProxy items to be indexed
        '''
        return cls.objects.none()


class SectionPageTags(Orderable):
    page = ParentalKey(SectionPage, related_name='section_tags')
    tag = models.ForeignKey(
        'wagtailcore.Page',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        help_text=_('Tags for tag navigation')
    )
    panels = [PageChooserPanel('tag', 'core.Tag')]
    api_fields = ['tag']


class ArticlePageTags(Orderable):
    page = ParentalKey(ArticlePage, related_name='nav_tags')
    tag = models.ForeignKey(
        'wagtailcore.Page',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        help_text=_('Tags for tag navigation')
    )
    panels = [PageChooserPanel('tag', 'core.Tag')]
    api_fields = ['tag']


class ArticlePageReactionQuestions(Orderable):
    page = ParentalKey(ArticlePage, related_name='reaction_questions')
    reaction_question = models.ForeignKey(
        'wagtailcore.Page',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        help_text=_('Reaction Questions')
    )
    panels = [PageChooserPanel('reaction_question', 'core.ReactionQuestion')]
    api_fields = ['reaction_question']


class ArticlePageRecommendedSections(Orderable):
    page = ParentalKey(ArticlePage, related_name='recommended_articles')
    recommended_article = models.ForeignKey(
        'wagtailcore.Page',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        help_text=_('Recommended articles for this article')
    )
    panels = [PageChooserPanel('recommended_article', 'core.ArticlePage')]
    api_fields = ['recommended_article']


class ArticlePageRelatedSections(Orderable):
    page = ParentalKey(ArticlePage, related_name='related_sections')
    section = models.ForeignKey(
        'wagtailcore.Page',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        help_text=_('Section that this page also belongs too')
    )
    panels = [PageChooserPanel('section', 'core.SectionPage')]
    api_fields = ['section']


class FooterIndexPage(MoloPage, PreventDeleteMixin):
    parent_page_types = []
    subpage_types = ['FooterPage']

    def copy(self, *args, **kwargs):
        site = kwargs['to'].get_site()
        main = site.root_page
        FooterIndexPage.objects.child_of(main).delete()
        super(FooterIndexPage, self).copy(*args, **kwargs)


class FooterPage(ArticlePage):
    parent_page_types = ['FooterIndexPage']
    subpage_types = []


FooterPage.content_panels = ArticlePage.content_panels
FooterPage.promote_panels = [
    MultiFieldPanel(
        Page.promote_panels,
        "Common page configuration", "collapsible collapsed")]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2017-12-20 11:48
from __future__ import unicode_literals

from django.db import migrations, models

from molo.core.models import Timezone

from pytz import common_timezones


def seed_timezones(apps, schema_editor):
    for timezone_title in common_timezones:
        timezone = Timezone.objects.create(title=timezone_title)
        timezone.save()


def delete_timezones(apps, schema_editor):
    Timezone.objects.all().delete()


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0074_banner_subtitle'),
    ]

    operations = [
        migrations.CreateModel(
            name='Timezone',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255, unique=True)),
            ],
        ),
        migrations.RunPython(seed_timezones, delete_timezones),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


def create_tag_index(apps, schema_editor):
    from molo.core.models import TagIndexPage, Main
    main = Main.objects.all().first()

    if main:
        tag_index = TagIndexPage(title='Tags', slug='tags')
        main.add_child(instance=tag_index)
        tag_index.save_revision().publish()


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0056_tag_navigation'),
    ]

    operations = [
        migrations.RunPython(create_tag_index),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2017-07-27 09:42
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0062_increase_char_length_reaction_success'),
    ]

    operations = [
        migrations.CreateModel(
            name='ArticlePageLanguageProxy',
            fields=[
            ],
            options={
                'verbose_name': 'Article View',
                'proxy': True,
                'verbose_name_plural': 'Article View',
            },
            bases=('core.articlepage',),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0012_add_featured_fields_to_articlepage'),
    ]

    operations = [
        migrations.AddField(
            model_name='articlepage',
            name='featured_in_homepage',
            field=models.BooleanField(default=False, help_text='Article to be featured in the Homepage within the Section module'),
        ),
    ]



from molo.profiles import views
from molo.profiles.forms import MoloAuthenticationForm

from django.conf.urls import url
from django.contrib.auth.decorators import login_required
from django.contrib.auth import views as auth_views
from django.views.generic import TemplateView


urlpatterns = [
    url(r'^logout/$', views.logout_page, name='auth_logout'),
    # If user is not login it will redirect to login page
    url(r'^login/$', auth_views.login,
        {'authentication_form': MoloAuthenticationForm}, name='auth_login'),
    url(
        r'^register/$',
        views.RegistrationView.as_view(),
        name='user_register'),
    url(
        r'^register/done/',
        login_required(views.RegistrationDone.as_view(
            template_name="profiles/done.html"
        )),
        name='registration_done'
    ),
    url(
        r'^view/myprofile/$',
        login_required(views.MyProfileView.as_view()),
        name='view_my_profile'
    ),
    url(
        r'^edit/myprofile/$',
        login_required(views.MyProfileEdit.as_view()),
        name='edit_my_profile'
    ),
    url(
        r'^password-reset/$',
        login_required(views.ProfilePasswordChangeView.as_view()),
        name="profile_password_change"
    ),
    url(
        r"^forgot-password/$",
        views.ForgotPasswordView.as_view(),
        name="forgot_password"
    ),
    url(
        r"^reset-password/$",
        views.ResetPasswordView.as_view(),
        name="reset_password"
    ),
    url(
        r"^reset-success/$",
        TemplateView.as_view(
            template_name="profiles/reset_password_success.html"
        ),
        name="reset_password_success"
    ),
    url(
        r"^login-success/$",
        TemplateView.as_view(
            template_name="profiles/login_success.html"
        ),
        name="login_success"
    ),
]



from django.test import TestCase, Client

from molo.core.models import Main, SiteLanguageRelation, Languages
from molo.core.tests.base import MoloTestCaseMixin
from molo.core.utils import generate_slug

from wagtail.wagtailcore.models import Site


class TestSites(TestCase, MoloTestCaseMixin):

    def setUp(self):
        self.mk_main()
        self.main = Main.objects.all().first()
        self.english = SiteLanguageRelation.objects.create(
            language_setting=Languages.for_site(self.main.get_site()),
            locale='en',
            is_active=True)
        self.yourmind = self.mk_section(
            self.section_index, title='Your mind')

    def test_two_sites_point_to_one_root_page(self):
        # assert that there is only one site rooted at main
        self.assertEquals(self.main.sites_rooted_here.count(), 1)
        client_1 = Client()
        # add a site that points to the same root page
        site_2 = Site.objects.create(
            hostname=generate_slug('site2'), port=80, root_page=self.main)
        # create a link buetween the current langauges and the new site
        Languages.objects.create(
            site_id=site_2.pk)
        SiteLanguageRelation.objects.create(
            language_setting=Languages.for_site(site_2),
            locale='en',
            is_active=True)
        client_2 = Client(HTTP_HOST=site_2.hostname)

        # assert that there are now two sites rooted at main
        self.assertEquals(self.main.sites_rooted_here.count(), 2)

        # assert that the correct hostname is returned for both sites
        response = client_1.get('/sections-main-1/your-mind/')
        self.assertEquals(response.status_code, 200)
        response = client_2.get(
            site_2.root_url + '/sections-main-1/your-mind/')
        self.assertEquals(response.status_code, 200)



from molo.core.tests.constants import TEST_IMAGE_HASH

AVAILABLE_ARTICLES = {
    "meta": {
        "total_count": 3
    },
    "items": [
        {
            "id": 10,
            "meta": {
                "type": "core.ArticlePage",
                "detail_url": "http://localhost:8000/api/v2/pages/10/",
            },
            "title": "Test article 1",
            "subtitle": "Sub for test article 1",
            "body": [
                {
                    "type": "paragraph",
                    "value": "Lorem ipsum dolor sit amet"
                },
                {
                    "type": "list",
                    "value": [
                        "Phasellus cursus eros turpis, vitae consequat sem."
                    ]
                },
                {
                    "type": "paragraph",
                    "value": "Lorem ipsum"
                }
            ],
            "tags": [],
            "commenting_state": "O",
            "commenting_open_time": "2016-11-11T06:00:00Z",
            "commenting_close_time": "2016-11-14T06:00:00Z",
            "social_media_title": "test image",
            "social_media_description": "test image description",
            "related_sections": [],
            "featured_in_latest": False,
            "featured_in_latest_start_date": "2016-11-11T06:00:00Z",
            "featured_in_latest_end_date": "2016-11-12T06:00:00Z",
            "featured_in_section": False,
            "featured_in_section_start_date": "2016-11-11T06:00:00Z",
            "featured_in_homepage": False,
            "featured_in_homepage_start_date": "2016-11-11T06:00:00Z",
            "featured_in_homepage_end_date": "2016-11-12T06:00:00Z",
            "feature_as_topic_of_the_day": True,
            "promote_date": "2016-11-11T06:00:00Z",
            "demote_date": "2016-11-14T06:00:00Z",
            "metadata_tags": [],
            "latest_revision_created_at": "2016-10-07T12:04:27.316423Z",
            "image": {
                "id": 1,
                "meta": {
                    "type": "wagtailimages.Image",
                    "detail_url": "http://localhost/api/v2/images/13/"
                }
            }
        },
        {
            "id": 11,
            "meta": {
                "type": "core.ArticlePage",
                "detail_url": "http://localhost:8000/api/v2/pages/11/",
            },
            "title": "Test article 2",
            "subtitle": "Sub for test article 2",
            "body": [
                {
                    "type": "paragraph",
                    "value": "Lorem ipsum dolor sit amet."
                },
                {
                    "type": "list",
                    "value": [
                        "Phasellus cursus eros turpis, vitae consequat sem "
                        "dapibus at. Sed fermentum mauris vitae fringilla "
                        "tristique. In hac habitasse platea dictumst."
                    ]
                },
                {
                    "type": "paragraph",
                    "value": "Lorem ipsum"
                }
            ],
            "tags": [
                "Another",
                "Test"
            ],
            "commenting_state": "O",
            "commenting_open_time": "2016-12-23T06:00:00Z",
            "commenting_close_time": "2016-12-26T06:00:00Z",
            "social_media_title": "test image",
            "social_media_description": "test image description",
            "related_sections": [
                {
                    "id": 1,
                    "meta": {
                        "type": "core.ArticlePageRelatedSections"
                    }
                }
            ],
            "featured_in_latest": False,
            "featured_in_latest_start_date": "2016-12-23T06:00:00Z",
            "featured_in_latest_end_date": "2016-12-24T06:00:00Z",
            "featured_in_section": False,
            "featured_in_section_start_date": "2016-12-23T06:00:00Z",
            "featured_in_homepage": False,
            "featured_in_homepage_start_date": "2016-12-20T06:00:00Z",
            "featured_in_homepage_end_date": "2016-12-21T06:00:00Z",
            "feature_as_topic_of_the_day": True,
            "promote_date": "2016-12-23T06:00:00Z",
            "demote_date": "2016-12-26T06:00:00Z",
            "metadata_tags": [],
            "latest_revision_created_at": "2016-11-09T10:17:45.352864Z",
            "image": {
                "id": 1,
                "meta": {
                    "type": "wagtailimages.Image",
                    "detail_url": "http://localhost/api/v2/images/17/"
                }
            }
        },
        {
            "id": 12,
            "meta": {
                "type": "core.ArticlePage",
                "detail_url": "http://localhost:8000/api/v2/pages/12/",
            },
            "title": "Test article 3",
            "subtitle": "Sub for test article 3",
            "body": [
                {
                    "type": "paragraph",
                    "value": "Lorem ipsum dolor sit amet."
                },
                {
                    "type": "list",
                    "value": [
                        "Phasellus cursus eros turpis, vitae consequat sem "
                        "dapibus at. Sed fermentum mauris vitae fringilla "
                        "tristique. In hac habitasse platea dictumst."
                    ]
                },
                {
                    "type": "paragraph",
                    "value": "Lorem ipsum"
                }
            ],
            "tags": [],
            "commenting_state": "O",
            "commenting_open_time": "2017-01-25T06:00:00Z",
            "commenting_close_time": "2017-01-27T06:00:00Z",
            "social_media_title": "test image",
            "social_media_description": "test image description",
            "related_sections": [],
            "featured_in_latest": False,
            "featured_in_latest_start_date": "2016-12-23T06:00:00Z",
            "featured_in_latest_end_date": "2016-12-24T06:00:00Z",
            "featured_in_section": False,
            "featured_in_section_start_date": "2016-12-23T06:00:00Z",
            "featured_in_homepage": False,
            "featured_in_homepage_start_date": "2016-12-23T06:00:00Z",
            "featured_in_homepage_end_date": "2016-12-26T06:00:00Z",
            "feature_as_topic_of_the_day": True,
            "promote_date": "2017-01-25T06:00:00Z",
            "demote_date": "2017-01-27T06:00:00Z",
            "metadata_tags": [],
            "latest_revision_created_at": "2016-10-10T11:04:36.153490Z",
            "image": {
                "id": 1,
                "meta": {
                    "type": "wagtailimages.Image",
                    "detail_url": "http://localhost/api/v2/images/60/"
                }
            }
        },
    ]
}

RELATED_IMAGE = {
    "id": 1,
    "meta": {
        "type": "wagtailimages.Image",
        "detail_url": "http://localhost/api/v2/images/1/",
        "tags": []
    },
    "title": "Image",
    "width": 480,
    "height": 480,
    "file": "http://localhost:8000/media/original_images/test.png"
}

AVAILABLE_SECTIONS = {
    "meta": {
        "total_count": 3
    },
    "items": [
        {
            "id": 2,
            "meta": {
                "type": "core.SectionPage",
                "detail_url": "http://localhost:8000/api/v2/pages/28/",
                "html_url": "http://localhost/sections/wellbeing/"
                            "taking-care-yourself/"
            },
            "title": "Taking care of yourself",
            "description": "",
            "image": {
                "id": 1,
                "meta": {
                    "type": "wagtailimages.Image",
                    "detail_url": "http://localhost:8000/api/v2/images/1/"
                }
            },
            "extra_style_hints": "",
            "commenting_state": "D",
            "commenting_open_time": "2017-01-25T06:00:00Z",
            "commenting_close_time": "2017-01-26T06:00:00Z",
            "time": [],
            "monday_rotation": False,
            "tuesday_rotation": False,
            "wednesday_rotation": False,
            "thursday_rotation": False,
            "friday_rotation": False,
            "saturday_rotation": False,
            "sunday_rotation": False,
            "content_rotation_start_date": "2017-01-25T06:00:00Z",
            "content_rotation_end_date": "2017-01-26T06:00:00Z",
            "latest_revision_created_at": "2016-10-04T10:23:59.504526Z"
        },
        {
            "id": 3,
            "meta": {
                "type": "core.SectionPage",
                "detail_url": "http://localhost:8000/api/v2/pages/3/",
            },
            "title": "Stress management",
            "description": "",
            "image": {
                "id": 1,
                "meta": {
                    "type": "wagtailimages.Image",
                    "detail_url": "http://localhost:8000/api/v2/images/1/"
                }
            },
            "extra_style_hints": "",
            "commenting_state": "D",
            "commenting_open_time": "2017-01-25T06:00:00Z",
            "commenting_close_time": "2017-01-26T06:00:00Z",
            "time": [],
            "monday_rotation": False,
            "tuesday_rotation": False,
            "wednesday_rotation": False,
            "thursday_rotation": False,
            "friday_rotation": False,
            "saturday_rotation": False,
            "sunday_rotation": False,
            "content_rotation_start_date": "2017-01-25T06:00:00Z",
            "content_rotation_end_date": "2017-01-26T06:00:00Z",
            "latest_revision_created_at": "2016-10-04T10:24:05.826271Z"
        },
        {
            "id": 4,
            "meta": {
                "type": "core.SectionPage",
                "detail_url": "http://localhost:8000/api/v2/pages/4/",
            },
            "title": "Breastfeeding",
            "description": "",
            "image": {
                "id": 1,
                "meta": {
                    "type": "wagtailimages.Image",
                    "detail_url": "http://localhost:8000/api/v2/images/1/"
                }
            },
            "extra_style_hints": "",
            "commenting_state": "D",
            "commenting_open_time": "2017-01-25T06:00:00Z",
            "commenting_close_time": "2017-01-26T06:00:00Z",
            "time": [],
            "monday_rotation": False,
            "tuesday_rotation": False,
            "wednesday_rotation": False,
            "thursday_rotation": False,
            "friday_rotation": False,
            "saturday_rotation": False,
            "sunday_rotation": False,
            "content_rotation_start_date": "2017-01-25T06:00:00Z",
            "content_rotation_end_date": "2017-01-26T06:00:00Z",
            "latest_revision_created_at": "2016-10-04T10:24:21.246246Z"
        },
    ]
}

AVAILABLE_SECTION_CHILDREN = {
    "meta": {
        "total_count": 2
    },
    "items": [
        {
            "id": 11,
            "meta": {
                "type": "core.ArticlePage",
                "detail_url": "http://localhost:8000/api/v2/pages/11/",
            },
            "title": "Test article 11"
        },
        {
            "id": 3,
            "meta": {
                "type": "core.SectionPage",
                "detail_url": "http://localhost:8000/api/v2/pages/3/",
            },
            "title": "Test section 3"
        },
    ]
}

LANGUAGE_LIST_RESPONSE = {
    "meta": {
        "total_count": 2
    },
    "items": [
        {
            "id": 1,
            "meta": {
                "type": "core.SiteLanguage",
                "detail_url": "http://localhost:8000/api/v2/languages/1/"
            }
        },
        {
            "id": 2,
            "meta": {
                "type": "core.SiteLanguage",
                "detail_url": "http://localhost:8000/api/v2/languages/2/"
            }
        }
    ]
}

LANGUAGE_RESPONSE_1 = {
    "id": 1,
    "meta": {
        "type": "core.SiteLanguage",
        "detail_url": "http://localhost:8000/api/v2/languages/1/"
    },
    "locale": "en",
    "is_main_language": True,
    "is_active": True
}

LANGUAGE_RESPONSE_2 = {
    "id": 2,
    "meta": {
        "type": "core.SiteLanguage",
        "detail_url": "http://localhost:8000/api/v2/languages/2/"
    },
    "locale": "fr",
    "is_main_language": False,
    "is_active": True
}

ARTICLE_PAGE_RESPONSE = {
    "id": 9999,
    "meta": {
        "type": "core.ArticlePage",
        "detail_url": "http://localhost:8000/api/v2/pages/12/",
        "html_url": "http://localhost:8000/sections/test-section/article-1/",
        "slug": "article-1",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-07-07T09:48:40.807381Z",
        "parent": {
            "id": 11,
            "meta": {
                "type": "core.SectionPage",
                "detail_url": "http://localhost:8000/api/v2/pages/11/",
                "html_url": "http://localhost:8000/sections/test-section/"
            },
            "title": "Test Section"
        },
        "children": None,
        "translations": [
            {
                "locale": "fr",
                "id": 13
            }
        ],
        "main_language_children": None
    },
    "title": "Article 1",
    "subtitle": "Subtitle for article 1",
    "body": [
        {
            "type": "paragraph",
            "value": (
                "Lorem ipsum dolor sit amet, consectetur adipiscing elit, "
                "sed do eiusmod tempor incididunt ut labore et dolore magna"
                " aliqua. Ut enim ad minim veniam, quis nostrud exercitation"
                " ullamco laboris nisi ut aliquip ex ea commodo consequat."
                " Duis aute irure dolor in reprehenderit in voluptate velit"
                " esse cillum dolore eu fugiat Nonea pariatur. Excepteur sint"
                " occaecat cupidatat non proident, sunt in culpa qui officia "
                "deserunt mollit anim id est laborum.")
        }
    ],
    "tags": [
        "tag1",
        "tag2",
        "tag3"
    ],
    "commenting_state": "D",
    "commenting_open_time": "2017-07-05T19:23:00Z",
    "commenting_close_time": "2016-09-29T20:00:00Z",
    "social_media_title": "Social Media",
    "social_media_description": "social_media_description",
    "social_media_image": {
        "id": 2,
        "meta": {
            "type": "wagtailimages.Image",
            "detail_url": "http://localhost:8000/api/v2/images/2/"
        },
        "title": "car"
    },
    "related_sections": [
        {
            "id": 1,
            "meta": {
                "type": "core.ArticlePageRelatedSections"
            },
            "section": {
                "id": 23,
                "meta": {
                    "type": "core.SectionPage",
                    "detail_url": "http://localhost:8000/api/v2/pages/23/"
                },
                "title": "Another Section"
            }
        },
        {
            "id": 2,
            "meta": {
                "type": "core.ArticlePageRelatedSections"
            },
            "section": {
                "id": 26,
                "meta": {
                    "type": "core.SectionPage",
                    "detail_url": "http://localhost:8000/api/v2/pages/26/"
                },
                "title": "Sub Section Test"
            }
        }
    ],
    "featured_in_latest": False,
    "featured_in_latest_start_date": "2017-07-16T18:17:04.642291Z",
    "featured_in_latest_end_date": "2017-07-16T18:17:04.642291Z",
    "featured_in_section": False,
    "featured_in_section_start_date": "2017-07-16T18:17:04.642291Z",
    "featured_in_section_end_date": "2017-07-16T18:17:04.642291Z",
    "featured_in_homepage": True,
    "featured_in_homepage_start_date": "2017-07-16T18:17:04.642291Z",
    "featured_in_homepage_end_date": "2017-07-16T18:17:04.642291Z",
    "feature_as_topic_of_the_day": False,
    "promote_date": "2017-07-16T18:17:04.642291Z",
    "demote_date": "2018-07-16T18:17:04.642291Z",
    "metadata_tags": [
        "metadata_tag1",
        "metadata_tag2"
    ],
    "latest_revision_created_at": "2017-07-16T18:17:04.642291Z",
    "image": {
        "id": 1,
        "meta": {
            "type": "wagtailimages.Image",
            "detail_url": "http://localhost:8000/api/v2/images/1/"
        },
        "title": "Mountain"
    },
    "reaction_questions": [
        {
            "id": 1,
            "meta": {
                "type": "core.ArticlePageReactionQuestions"
            },
            "reaction_question": {
                "id": 37,
                "meta": {
                    "type": "core.ReactionQuestion",
                    "detail_url": "http://localhost:8000/api/v2/pages/37/"
                },
                "title": "How does this make you feel?"
            }
        },
        {
            "id": 2,
            "meta": {
                "type": "core.ArticlePageReactionQuestions"
            },
            "reaction_question": {
                "id": 38,
                "meta": {
                    "type": "core.ReactionQuestion",
                    "detail_url": "http://localhost:8000/api/v2/pages/38/"
                },
                "title": "What colour was the dog?"
            }
        }
    ],
    "nav_tags": [
        {
            "id": 1,
            "meta": {
                "type": "core.ArticlePageTags"
            },
            "tag": {
                "id": 35,
                "meta": {
                    "type": "core.Tag",
                    "detail_url": "http://localhost:8000/api/v2/pages/35/"
                },
                "title": "NAV TAG 1"
            }
        }
    ],
    "recommended_articles": [
        {
            "id": 1,
            "meta": {
                "type": "core.ArticlePageRecommendedSections"
            },
            "recommended_article": {
                "id": 27,
                "meta": {
                    "type": "core.ArticlePage",
                    "detail_url": "http://localhost:8000/api/v2/pages/27/"
                },
                "title": "Article that is nested"
            }
        },
        {
            "id": 2,
            "meta": {
                "type": "core.ArticlePageRecommendedSections"
            },
            "recommended_article": {
                "id": 22,
                "meta": {
                    "type": "core.ArticlePage",
                    "detail_url": "http://localhost:8000/api/v2/pages/22/"
                },
                "title": "Article to Import 1"
            }
        }
    ],
    "go_live_at": "2017-08-01T13:23:00Z",
    "expire_at": "2017-08-31T13:23:00Z",
    "expired": False,
}

ARTICLE_PAGE_RESPONSE_STREAM_FIELDS = {
    "id": 92,
    "meta": {
        "type": "core.ArticlePage",
        "detail_url": "http://localhost:9000/api/v2/pages/92/",
        "html_url": "http://localhost:9000/sections/test-section/article-all-stream-fields/",  # noqa
        "slug": "article-all-stream-fields",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-08-23T08:56:14.263738Z",
        "parent": {
            "id": 11,
            "meta": {
                "type": "core.SectionPage",
                "detail_url": "http://localhost:9000/api/v2/pages/11/",
                "html_url": "http://localhost:9000/sections/test-section/"
            },
            "title": "Test Section"
        },
        "children": None,
        "translations": [],
        "main_language_children": None
    },
    "title": "ARTICLE WITH ALL THE STREAM FIELDS",
    "subtitle": "",
    "body": [
        {
            "type": "heading",
            "value": "test heading"
        },
        {
            "type": "paragraph",
            "value": "test paragraph"
        },
        {
            "type": "image",
            "value": 297
        },
        {
            "type": "list",
            "value": [
                "list item 1",
                "list item 2",
                "list item 3"
            ]
        },
        {
            "type": "numbered_list",
            "value": [
                "numbered list 1",
                "numbered list 2",
                "numbered list 3"
            ]
        },
        {
            "type": "page",
            "value": 48
        }
    ],
    "tags": [],
    "commenting_state": None,
    "commenting_open_time": None,
    "commenting_close_time": None,
    "social_media_title": "",
    "social_media_description": "",
    "social_media_image": None,
    "related_sections": [],
    "featured_in_latest": False,
    "featured_in_latest_start_date": None,
    "featured_in_latest_end_date": None,
    "featured_in_section": False,
    "featured_in_section_start_date": None,
    "featured_in_section_end_date": None,
    "featured_in_homepage": False,
    "featured_in_homepage_start_date": None,
    "featured_in_homepage_end_date": None,
    "feature_as_topic_of_the_day": False,
    "promote_date": None,
    "demote_date": None,
    "metadata_tags": [],
    "latest_revision_created_at": "2017-08-23T08:57:13.824364Z",
    "image": None,
    "reaction_questions": [],
    "nav_tags": [],
    "recommended_articles": []
}


SECTION_PAGE_RESPONSE = {
    "id": 11,
    "meta": {
        "type": "core.SectionPage",
        "detail_url": "http://localhost:8000/api/v2/pages/11/",
        "html_url": "http://localhost:8000/sections/test-section/",
        "slug": "test-section",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-07-07T09:47:34.122769Z",
        "parent": {
            "id": 6,
            "meta": {
                "type": "core.SectionIndexPage",
                "detail_url": "http://localhost:8000/api/v2/pages/6/",
                "html_url": "http://localhost:8000/sections/"
            },
            "title": "Sections"
        },
        "children": {
            "items": [
                12,
                13,
                22,
                26,
                34
            ]
        },
        "translations": [
            {
                "locale": "ve",
                "id": 24
            },
            {
                "locale": "fr",
                "id": 25
            }
        ],
        "main_language_children": [
            12,
            22,
            26,
            34
        ]
    },
    "title": "Test Section",
    "live": True,
    "description": "section description",
    "image": {
        "id": 1,
        "meta": {
            "type": "wagtailimages.Image",
            "detail_url": "http://localhost:8000/api/v2/images/1/"
        },
        "title": "Mountain"
    },
    "extra_style_hints": "blue",
    "commenting_state": "O",
    "commenting_open_time": "2017-07-17T10:00:00Z",
    "commenting_close_time": "2017-08-29T11:07:00Z",
    "time": [
        {
            "type": "time",
            "value": "16:00:00"
        },
        {
            "type": "time",
            "value": "15:00:00"
        }
    ],
    "monday_rotation": True,
    "tuesday_rotation": False,
    "wednesday_rotation": False,
    "thursday_rotation": False,
    "friday_rotation": True,
    "saturday_rotation": False,
    "sunday_rotation": False,
    "content_rotation_start_date": "2017-07-04T13:55:00Z",
    "content_rotation_end_date": "2017-07-17T13:55:00Z",
    "latest_revision_created_at": "2017-07-16T18:04:03.439357Z",
    "section_tags": [
        {
            "id": 1,
            "meta": {
                "type": "core.SectionPageTags"
            },
            "tag": {
                "id": 36,
                "meta": {
                    "type": "core.Tag",
                    "detail_url": "http://localhost:8000/api/v2/pages/36/"
                },
                "title": "NAV TAG 2"
            }
        },
        {
            "id": 2,
            "meta": {
                "type": "core.SectionPageTags"
            },
            "tag": {
                "id": 35,
                "meta": {
                    "type": "core.Tag",
                    "detail_url": "http://localhost:8000/api/v2/pages/35/"
                },
                "title": "NAV TAG 1"
            }
        }
    ],
    "enable_next_section": True,
    "enable_recommended_section": True,
    "go_live_at": "2017-08-09T16:05:00Z",
    "expire_at": "2017-08-26T16:05:00Z",
    "expired": False
}

BANNER_SITE_RESPONSE = {
    "id": 40,
    "meta": {
        "type": "core.BannerPage",
        "detail_url": "http://localhost:8000/api/v2/pages/40/",
        "html_url": "http://localhost:8000/banners/banner-1/",
        "slug": "banner-1",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-07-17T14:04:05.433345Z",
        "parent": {
            "id": 5,
            "meta": {
                "type": "core.BannerIndexPage",
                "detail_url": "http://localhost:8000/api/v2/pages/5/",
                "html_url": "http://localhost:8000/banners/"
            },
            "title": "Banners"
        },
        "children": None,
        "translations": [],
        "main_language_children": None
    },
    "title": "Banner 1",
    "banner": {
        "id": 1,
        "meta": {
            "type": "wagtailimages.Image",
            "detail_url": "http://localhost:8000/api/v2/images/1/"
        },
        "title": "Mountain"
    },
    "banner_link_page": {
        "id": 13,
        "meta": {
            "type": "core.ArticlePage",
            "detail_url": "http://localhost:8000/api/v2/pages/13/"
        },
        "title": "French translation of Article 1"
    },
    "external_link": "https://www.google.co.za/"
}

WAGTAIL_API_LIST_VIEW = {
    "meta": {
        "total_count": 3
    },
    "items": [1, 2, 3]
}

WAGTAIL_API_LIST_VIEW_PAGE_1 = {
    "meta": {
        "total_count": 25
    },
    "items": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10,
              11, 12, 13, 14, 15, 16, 17, 18, 19, 20]
}

WAGTAIL_API_LIST_VIEW_PAGE_2 = {
    "meta": {
        "total_count": 25
    },
    "items": [21, 22, 23, 24, 25]
}

IMAGE_LIST_RESPONSE = {
    "meta": {
        "total_count": 2
    },
    "items": [
        {
            "id": 97,
            "meta": {
                "type": "wagtailimages.Image",
                "detail_url": "http://localhost:8000/api/v2/images/97/",
                "tags": []
            },
            "title": "GAO_A4qQsiM.jpg"
        },
        {
            "id": 98,
            "meta": {
                "type": "wagtailimages.Image",
                "detail_url": "http://localhost:8000/api/v2/images/98/",
                "tags": []
            },
            "title": "boop.png"
        }
    ]
}

IMAGE_DETAIL_1 = {
    "id": 97,
    "meta": {
        "type": "wagtailimages.Image",
        "detail_url": "http://localhost:8000/api/v2/images/97/",
        "tags": []
    },
    "title": "GAO_A4qQsiM.jpg",
    "width": 640,
    "height": 480,
    "filename": "GAO_A4qQsiM.jpg",
    "file": "http://localhost:8000/media/original_images/GAO_A4qQsiM.jpg",
    "image_url": "/media/images/GAO_A4qQsiM.original.jpg",
    "image_hash": TEST_IMAGE_HASH
}

IMAGE_DETAIL_2 = {
    "id": 98,
    "meta": {
        "type": "wagtailimages.Image",
        "detail_url": "http://localhost:8000/api/v2/images/98/",
        "tags": []
    },
    "title": "uYh_80lp8.png",
    "width": 1080,
    "height": 644,
    "filename": "uYh_80lp8.png",
    "file": "http://localhost:8000/media/original_images/uYh_80lp8.png",
    "image_url": "/media/images/uYh_80lp8.original.jpg",
    "image_hash": '3f7ffffffee00000'
}

IMAGE_DETAIL_1_NO_HASH = {
    "id": 97,
    "meta": {
        "type": "wagtailimages.Image",
        "detail_url": "http://localhost:8000/api/v2/images/97/",
        "tags": []
    },
    "title": "GAO_A4qQsiM.jpg",
    "width": 640,
    "height": 480,
    "filename": "GAO_A4qQsiM.jpg",
    "file": "http://localhost:8000/media/original_images/GAO_A4qQsiM.jpg",
    "image_url": "/media/images/GAO_A4qQsiM.original.jpg",
    "image_hash": None
}

ARTICLE_PAGE_RESPONSE_MAIN_LANG = {
    "id": 12,
    "meta": {
        "type": "core.ArticlePage",
        "detail_url": "http://localhost:8000/api/v2/pages/12/",
        "html_url": "http://localhost:8000/sections/test-section/article-1/",
        "slug": "article-1",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-07-07T09:48:40.807381Z",
        "parent": {
            "id": 11,
            "meta": {
                "type": "core.SectionPage",
                "detail_url": "http://localhost:8000/api/v2/pages/11/",
                "html_url": "http://localhost:8000/sections/test-section/"
            },
            "title": "Test Section"
        },
        "children": None,
        "translations": [
            {
                "locale": "fr",
                "id": 2
            }
        ],
        "main_language_children": None
    },
    "title": "Article 1",
    "subtitle": "Subtitle for article 1",
    "body": [
        {
            "type": "paragraph",
            "value": "Content of Article 1 "
        }
    ],
    "tags": [],
    "commenting_state": None,
    "commenting_open_time": None,
    "commenting_close_time": None,
    "social_media_title": "",
    "social_media_description": "",
    "social_media_image": None,
    "related_sections": [],
    "featured_in_latest": False,
    "featured_in_latest_start_date": None,
    "featured_in_latest_end_date": None,
    "featured_in_section": False,
    "featured_in_section_start_date": None,
    "featured_in_section_end_date": None,
    "featured_in_homepage": False,
    "featured_in_homepage_start_date": None,
    "featured_in_homepage_end_date": None,
    "feature_as_topic_of_the_day": False,
    "promote_date": None,
    "demote_date": None,
    "metadata_tags": [],
    "latest_revision_created_at": "2017-07-10T08:52:14.326306Z",
    "image": None,
    "reaction_questions": [],
    "nav_tags": [],
    "recommended_articles": []
}

ARTICLE_PAGE_RESPONSE_FRENCH = {
    "id": 13,
    "meta": {
        "type": "core.ArticlePage",
        "detail_url": "http://localhost:8000/api/v2/pages/13/",
        "html_url": "http://localhost:8000/sections/test-section/french-translation-of-article-1/",  # noqa
        "slug": "french-translation-of-article-1",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-07-10T08:52:14.349610Z",
        "parent": {
            "id": 11,
            "meta": {
                "type": "core.SectionPage",
                "detail_url": "http://localhost:8000/api/v2/pages/11/",
                "html_url": "http://localhost:8000/sections/test-section/"
            },
            "title": "Test Section"
        },
        "children": None,
        "translations": [
            {
                "locale": "en",
                "id": 12
            }
        ],
        "main_language_children": None
    },
    "title": "French translation of Article 1",
    "subtitle": "",
    "body": [
        {
            "type": "paragraph",
            "value": "article 1 content in french"
        }
    ],
    "tags": [],
    "commenting_state": None,
    "commenting_open_time": None,
    "commenting_close_time": None,
    "social_media_title": "",
    "social_media_description": "",
    "social_media_image": None,
    "related_sections": [],
    "featured_in_latest": False,
    "featured_in_latest_start_date": None,
    "featured_in_latest_end_date": None,
    "featured_in_section": False,
    "featured_in_section_start_date": None,
    "featured_in_section_end_date": None,
    "featured_in_homepage": False,
    "featured_in_homepage_start_date": None,
    "featured_in_homepage_end_date": None,
    "feature_as_topic_of_the_day": False,
    "promote_date": None,
    "demote_date": None,
    "metadata_tags": [],
    "latest_revision_created_at": "2017-07-10T08:52:14.326306Z",
    "image": None,
    "reaction_questions": [],
    "nav_tags": [],
    "recommended_articles": []
}

BANNER_PAGE_RESPONSE = {
    "id": 40,
    "meta": {
        "type": "core.BannerPage",
        "detail_url": "http://localhost:8000/api/v2/pages/40/",
        "html_url": "http://localhost:8000/banners/banner-1/",
        "slug": "banner-1",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-07-17T14:04:05.433345Z",
        "parent": {
            "id": 5,
            "meta": {
                "type": "core.BannerIndexPage",
                "detail_url": "http://localhost:8000/api/v2/pages/5/",
                "html_url": "http://localhost:8000/banners/"
            },
            "title": "Banners"
        },
        "children": None,
        "translations": [],
        "main_language_children": None
    },
    "title": "Banner 1",
    "banner": {
        "id": 97,
        "meta": {
            "type": "wagtailimages.Image",
            "detail_url": "http://localhost:8000/api/v2/images/97/"
        },
        "title": "GAO_A4qQsiM.jpg"
    },
    "banner_link_page": {
        "id": 13,
        "meta": {
            "type": "core.ArticlePage",
            "detail_url": "http://localhost:8000/api/v2/pages/13/"
        },
        "title": "French translation of Article 1"
    },
    "external_link": "https://www.google.co.za/"
}

TAG_PAGE_RESPONSE = {
    "id": 35,
    "meta": {
        "type": "core.Tag",
        "detail_url": "http://localhost:8000/api/v2/pages/35/",
        "html_url": "http://localhost:8000/tags/nav-tag-1/",
        "slug": "nav-tag-1",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-07-16T17:51:43.179465Z",
        "parent": {
            "id": 8,
            "meta": {
                "type": "core.TagIndexPage",
                "detail_url": "http://localhost:8000/api/v2/pages/8/",
                "html_url": "http://localhost:8000/tags/"
            },
            "title": "Tags"
        },
        "children": None,
        "translations": [],
        "main_language_children": None
    },
    "title": "NAV TAG 1",
    "feature_in_homepage": False,
    "go_live_at": "2017-09-01T16:48:00Z",
    "expire_at": "2017-09-19T16:48:00Z",
    "expired": False
}

SECTION_PAGE_RESPONSE_FRENCH = {
    "id": 25,
    "meta": {
        "type": "core.SectionPage",
        "detail_url": "http://localhost:8000/api/v2/pages/25/",
        "html_url": "http://localhost:8000/sections/french-translation-of-test-section/",  # noqa
        "slug": "french-translation-of-test-section",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-07-13T10:31:18.587366Z",
        "parent": {
            "id": 6,
            "meta": {
                "type": "core.SectionIndexPage",
                "detail_url": "http://localhost:8000/api/v2/pages/6/",
                "html_url": "http://localhost:8000/sections/"
            },
            "title": "Sections"
        },
        "children": None,
        "translations": [
            {
                "locale": "en",
                "id": 11
            },
            {
                "locale": "ve",
                "id": 24
            }
        ],
        "main_language_children": None
    },
    "title": "French translation of Test Section",
    "live": True,
    "description": "",
    "image": None,
    "extra_style_hints": "",
    "commenting_state": None,
    "commenting_open_time": None,
    "commenting_close_time": None,
    "time": [],
    "monday_rotation": False,
    "tuesday_rotation": False,
    "wednesday_rotation": False,
    "thursday_rotation": False,
    "friday_rotation": False,
    "saturday_rotation": False,
    "sunday_rotation": False,
    "content_rotation_start_date": None,
    "content_rotation_end_date": None,
    "section_tags": []
}

TYPE_SECTION_INDEX_PAGE_RESPONSE = {
    "meta": {
        "total_count": 1
    },
    "items": [
        {
            "id": 6,
            "meta": {
                "type": "core.SectionIndexPage",
                "detail_url": "http://localhost:8000/api/v2/pages/6/",
                "html_url": "http://localhost:8000/sections/",
                "slug": "sections",
                "first_published_at": "2017-07-06T14:35:24.673565Z"
            },
            "title": "Sections"
        }
    ]
}

# START
# Integration Test for Recursive Import
# http://localhost:8000/api/v2/pages/6/
SECTION_INDEX_PAGE_RESPONSE = {
    "id": 6,
    "meta": {
        "type": "core.SectionIndexPage",
        "detail_url": "http://localhost:8000/api/v2/pages/6/",
        "html_url": "http://localhost:8000/sections/",
        "slug": "sections",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-07-07T10:48:23.219970Z",
        "parent": {
            "id": 4,
            "meta": {
                "type": "core.Main",
                "detail_url": "http://localhost:8000/api/v2/pages/4/",
                "html_url": "http://localhost:8000/"
            },
            "title": "Main"
        },
        "children": {
            "items": [
                178,
                179,
                180
            ]
        },
        "translations": None,
        "main_language_children": [
            178,
            179
        ]
    },
    "title": "Sections"
}

# First Section Imported
# http://localhost:8000/api/v2/pages/178/
SECTION_RESPONSE_1 = {
    "id": 178,
    "meta": {
        "type": "core.SectionPage",
        "detail_url": "http://localhost:8000/api/v2/pages/178/",
        "html_url": "http://localhost:8000/sections/section-1/",
        "slug": "section-1",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-08-04T10:07:03.947926Z",
        "parent": {
            "id": 6,
            "meta": {
                "type": "core.SectionIndexPage",
                "detail_url": "http://localhost:8000/api/v2/pages/6/",
                "html_url": "http://localhost:8000/sections/"
            },
            "title": "Sections"
        },
        "children": {
            "items": [
                181,
                182,
                183
            ]
        },
        "translations": [
            {
                "locale": "fr",
                "id": 180
            }
        ],
        "main_language_children": [
            181,
            182
        ]
    },
    "title": "Section 1",
    "live": True,
    "description": "Description for Site 1",
    "image": None,
    "extra_style_hints": "",
    "commenting_state": None,
    "commenting_open_time": None,
    "commenting_close_time": None,
    "time": [],
    "monday_rotation": False,
    "tuesday_rotation": False,
    "wednesday_rotation": False,
    "thursday_rotation": False,
    "friday_rotation": False,
    "saturday_rotation": False,
    "sunday_rotation": False,
    "content_rotation_start_date": None,
    "content_rotation_end_date": None,
    "section_tags": []
}

# Translation of Section 1
# http://localhost:8000/api/v2/pages/180/
SECTION_RESPONSE_1_TRANSLATION_1 = {
    "id": 180,
    "meta": {
        "type": "core.SectionPage",
        "detail_url": "http://localhost:8000/api/v2/pages/180/",
        "html_url": "http://localhost:8000/sections/french-translation-of-section-1/",  # noqa
        "slug": "french-translation-of-section-1",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-08-04T10:07:28.164537Z",
        "parent": {
            "id": 6,
            "meta": {
                "type": "core.SectionIndexPage",
                "detail_url": "http://localhost:8000/api/v2/pages/6/",
                "html_url": "http://localhost:8000/sections/"
            },
            "title": "Sections"
        },
        "children": None,
        "translations": [
            {
                "locale": "en",
                "id": 178
            }
        ],
        "main_language_children": None
    },
    "title": "French translation of Section 1",
    "live": True,
    "description": "",
    "image": None,
    "extra_style_hints": "",
    "commenting_state": None,
    "commenting_open_time": None,
    "commenting_close_time": None,
    "time": [],
    "monday_rotation": False,
    "tuesday_rotation": False,
    "wednesday_rotation": False,
    "thursday_rotation": False,
    "friday_rotation": False,
    "saturday_rotation": False,
    "sunday_rotation": False,
    "content_rotation_start_date": None,
    "content_rotation_end_date": None,
    "section_tags": []
}

# Article Page
# http://localhost:8000/api/v2/pages/181/
ARTICLE_RESPONSE_1 = {
    "id": 181,
    "meta": {
        "type": "core.ArticlePage",
        "detail_url": "http://localhost:8000/api/v2/pages/181/",
        "html_url": "http://localhost:8000/sections/section-1/article-1/",
        "slug": "article-1",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-08-04T10:07:58.625958Z",
        "parent": {
            "id": 178,
            "meta": {
                "type": "core.SectionPage",
                "detail_url": "http://localhost:8000/api/v2/pages/178/",
                "html_url": "http://localhost:8000/sections/section-1/"
            },
            "title": "Section 1"
        },
        "children": None,
        "translations": [
            {
                "locale": "fr",
                "id": 183
            }
        ],
        "main_language_children": None
    },
    "title": "Article 1",
    "subtitle": "",
    "body": [],
    "tags": [],
    "commenting_state": None,
    "commenting_open_time": None,
    "commenting_close_time": None,
    "social_media_title": "",
    "social_media_description": "",
    "social_media_image": None,
    "related_sections": [],
    "featured_in_latest": False,
    "featured_in_latest_start_date": None,
    "featured_in_latest_end_date": None,
    "featured_in_section": False,
    "featured_in_section_start_date": None,
    "featured_in_section_end_date": None,
    "featured_in_homepage": False,
    "featured_in_homepage_start_date": None,
    "featured_in_homepage_end_date": None,
    "feature_as_topic_of_the_day": False,
    "promote_date": None,
    "demote_date": None,
    "metadata_tags": [],
    "latest_revision_created_at": "2017-08-04T10:07:58.605558Z",
    "image": None,
    "reaction_questions": [],
    "nav_tags": [],
    "recommended_articles": []
}

# http://localhost:8000/api/v2/pages/183/
ARTICLE_RESPONSE_1_TRANSLATION = {
    "id": 183,
    "meta": {
        "type": "core.ArticlePage",
        "detail_url": "http://localhost:8000/api/v2/pages/183/",
        "html_url": "http://localhost:8000/sections/section-1/french-translation-of-article-1-2/",  # noqa
        "slug": "french-translation-of-article-1-2",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-08-04T10:08:14.444885Z",
        "parent": {
            "id": 178,
            "meta": {
                "type": "core.SectionPage",
                "detail_url": "http://localhost:8000/api/v2/pages/178/",
                "html_url": "http://localhost:8000/sections/section-1/"
            },
            "title": "Section 1"
        },
        "children": None,
        "translations": [
            {
                "locale": "en",
                "id": 181
            }
        ],
        "main_language_children": None
    },
    "title": "French translation of Article 1",
    "subtitle": "",
    "body": [],
    "tags": [],
    "commenting_state": None,
    "commenting_open_time": None,
    "commenting_close_time": None,
    "social_media_title": "",
    "social_media_description": "",
    "social_media_image": None,
    "related_sections": [],
    "featured_in_latest": False,
    "featured_in_latest_start_date": None,
    "featured_in_latest_end_date": None,
    "featured_in_section": False,
    "featured_in_section_start_date": None,
    "featured_in_section_end_date": None,
    "featured_in_homepage": False,
    "featured_in_homepage_start_date": None,
    "featured_in_homepage_end_date": None,
    "feature_as_topic_of_the_day": False,
    "promote_date": None,
    "demote_date": None,
    "metadata_tags": [],
    "latest_revision_created_at": "2017-08-04T10:08:14.425964Z",
    "image": None,
    "reaction_questions": [],
    "nav_tags": [],
    "recommended_articles": []
}

# http://localhost:8000/api/v2/pages/182/
ARTICLE_RESPONSE_2 = {
    "id": 182,
    "meta": {
        "type": "core.ArticlePage",
        "detail_url": "http://localhost:8000/api/v2/pages/182/",
        "html_url": "http://localhost:8000/sections/section-1/article-2/",
        "slug": "article-2",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-08-04T10:08:08.877299Z",
        "parent": {
            "id": 178,
            "meta": {
                "type": "core.SectionPage",
                "detail_url": "http://localhost:8000/api/v2/pages/178/",
                "html_url": "http://localhost:8000/sections/section-1/"
            },
            "title": "Section 1"
        },
        "children": None,
        "translations": [],
        "main_language_children": None
    },
    "title": "Article 2",
    "subtitle": "",
    "body": [],
    "tags": [],
    "commenting_state": None,
    "commenting_open_time": None,
    "commenting_close_time": None,
    "social_media_title": "",
    "social_media_description": "",
    "social_media_image": None,
    "related_sections": [],
    "featured_in_latest": False,
    "featured_in_latest_start_date": None,
    "featured_in_latest_end_date": None,
    "featured_in_section": False,
    "featured_in_section_start_date": None,
    "featured_in_section_end_date": None,
    "featured_in_homepage": False,
    "featured_in_homepage_start_date": None,
    "featured_in_homepage_end_date": None,
    "feature_as_topic_of_the_day": False,
    "promote_date": None,
    "demote_date": None,
    "metadata_tags": [],
    "latest_revision_created_at": "2017-08-04T10:08:08.854104Z",
    "image": None,
    "reaction_questions": [],
    "nav_tags": [],
    "recommended_articles": []
}

# http://localhost:8000/api/v2/pages/179/
SECTION_RESPONSE_2 = {
    "id": 179,
    "meta": {
        "type": "core.SectionPage",
        "detail_url": "http://localhost:8000/api/v2/pages/179/",
        "html_url": "http://localhost:8000/sections/section-2/",
        "slug": "section-2",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-08-04T10:07:14.463717Z",
        "parent": {
            "id": 6,
            "meta": {
                "type": "core.SectionIndexPage",
                "detail_url": "http://localhost:8000/api/v2/pages/6/",
                "html_url": "http://localhost:8000/sections/"
            },
            "title": "Sections"
        },
        "children": {
            "items": [
                184
            ]
        },
        "translations": [],
        "main_language_children": [
            184
        ]
    },
    "title": "Section 2",
    "live": True,
    "description": "Description 2",
    "image": None,
    "extra_style_hints": "",
    "commenting_state": None,
    "commenting_open_time": None,
    "commenting_close_time": None,
    "time": [],
    "monday_rotation": False,
    "tuesday_rotation": False,
    "wednesday_rotation": False,
    "thursday_rotation": False,
    "friday_rotation": False,
    "saturday_rotation": False,
    "sunday_rotation": False,
    "content_rotation_start_date": None,
    "content_rotation_end_date": None,
    "section_tags": []
}

# http://localhost:8000/api/v2/pages/184/
SUB_SECTION_RESPONSE_1 = {
    "id": 184,
    "meta": {
        "type": "core.SectionPage",
        "detail_url": "http://localhost:8000/api/v2/pages/184/",
        "html_url": "http://localhost:8000/sections/section-2/sub-section/",
        "slug": "sub-section",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-08-04T10:08:47.727354Z",
        "parent": {
            "id": 179,
            "meta": {
                "type": "core.SectionPage",
                "detail_url": "http://localhost:8000/api/v2/pages/179/",
                "html_url": "http://localhost:8000/sections/section-2/"
            },
            "title": "Section 2"
        },
        "children": {
            "items": [
                185
            ]
        },
        "translations": [],
        "main_language_children": [
            185
        ]
    },
    "title": "Sub Section",
    "live": True,
    "description": "",
    "image": None,
    "extra_style_hints": "",
    "commenting_state": None,
    "commenting_open_time": None,
    "commenting_close_time": None,
    "time": [],
    "monday_rotation": False,
    "tuesday_rotation": False,
    "wednesday_rotation": False,
    "thursday_rotation": False,
    "friday_rotation": False,
    "saturday_rotation": False,
    "sunday_rotation": False,
    "content_rotation_start_date": None,
    "content_rotation_end_date": None,
    "section_tags": []
}

# http://localhost:8000/api/v2/pages/185/
NESTED_ARTICLE_RESPONSE = {
    "id": 185,
    "meta": {
        "type": "core.ArticlePage",
        "detail_url": "http://localhost:8000/api/v2/pages/185/",
        "html_url": "http://localhost:8000/sections/section-2/sub-section/article-3/",  # noqa
        "slug": "article-3",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-08-04T10:09:01.177880Z",
        "parent": {
            "id": 184,
            "meta": {
                "type": "core.SectionPage",
                "detail_url": "http://localhost:8000/api/v2/pages/184/",
                "html_url": "http://localhost:8000/sections/section-2/sub-section/"  # noqa
            },
            "title": "Sub Section"
        },
        "children": None,
        "translations": [],
        "main_language_children": None
    },
    "title": "Article 3",
    "subtitle": "",
    "body": [],
    "tags": [],
    "commenting_state": None,
    "commenting_open_time": None,
    "commenting_close_time": None,
    "social_media_title": "",
    "social_media_description": "",
    "social_media_image": None,
    "related_sections": [],
    "featured_in_latest": False,
    "featured_in_latest_start_date": None,
    "featured_in_latest_end_date": None,
    "featured_in_section": False,
    "featured_in_section_start_date": None,
    "featured_in_section_end_date": None,
    "featured_in_homepage": False,
    "featured_in_homepage_start_date": None,
    "featured_in_homepage_end_date": None,
    "feature_as_topic_of_the_day": False,
    "promote_date": None,
    "demote_date": None,
    "metadata_tags": [],
    "latest_revision_created_at": "2017-08-04T10:09:01.158928Z",
    "image": None,
    "reaction_questions": [],
    "nav_tags": [],
    "recommended_articles": []
}

NESTED_FIELD_NAV_TAG_WITH_NONE = {
    "nav_tags": [
        {
            "id": 10,
            "meta": {
                "type": "core.ArticlePageTags"
            },
            "tag": None
        },
        {
            "id": 11,
            "meta": {
                "type": "core.ArticlePageTags"
            },
            "tag": {
                "id": 28170,
                "meta": {
                    "type": "core.Tag",
                    "detail_url": "http://qa.tuneme.seed.p16n.org/api/v2/pages/28170/"  # noqa
                },
                "title": "18"
            }
        }
    ]
}

ARTICLE_WITH_ONLY_IMAGE_RESPONSE = {
    "id": 93,
    "meta": {
        "type": "core.ArticlePage",
        "detail_url": "http://localhost:9000/api/v2/pages/93/",
        "html_url": "http://localhost:9000/sections/test-section/bare-article/",  # noqa
        "slug": "bare-article",
        "show_in_menus": False,
        "seo_title": "",
        "search_description": "",
        "first_published_at": "2017-08-30T12:45:06.398516Z",
        "parent": {
            "id": 11,
            "meta": {
                "type": "core.SectionPage",
                "detail_url": "http://localhost:9000/api/v2/pages/11/",
                "html_url": "http://localhost:9000/sections/test-section/"
            },
            "title": "Test Section"
        },
        "children": None,
        "translations": [],
        "main_language_children": None
    },
    "title": "Bare Article",
    "subtitle": "",
    "body": [],
    "tags": [],
    "commenting_state": None,
    "commenting_open_time": None,
    "commenting_close_time": None,
    "social_media_title": "",
    "social_media_description": "",
    "social_media_image": None,
    "related_sections": [],
    "featured_in_latest": False,
    "featured_in_latest_start_date": None,
    "featured_in_latest_end_date": None,
    "featured_in_section": False,
    "featured_in_section_start_date": None,
    "featured_in_section_end_date": None,
    "featured_in_homepage": False,
    "featured_in_homepage_start_date": None,
    "featured_in_homepage_end_date": None,
    "feature_as_topic_of_the_day": False,
    "promote_date": None,
    "demote_date": None,
    "metadata_tags": [],
    "latest_revision_created_at": "2017-08-30T12:45:06.378020Z",
    "image": {
        "id": 297,
        "meta": {
            "type": "wagtailimages.Image",
            "detail_url": "http://localhost:9000/api/v2/images/297/"
        },
        "title": "and it's more than just attraction."
    },
    "reaction_questions": [],
    "nav_tags": [],
    "recommended_articles": [],
    "go_live_at": None,
    "expire_at": None,
    "expired": False
}



# -*- coding: utf-8 -*-
# Generated by Django 1.9.11 on 2016-11-17 08:24
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0047_add_core_permissions_to_groups'),
    ]

    operations = [
        migrations.AddField(
            model_name='articlepage',
            name='featured_in_homepage_end_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='featured_in_homepage_start_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='featured_in_latest_end_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='featured_in_latest_start_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='featured_in_section_end_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='articlepage',
            name='featured_in_section_start_date',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0015_add_more_verbose_names'),
        ('wagtailimages', '0006_add_verbose_names'),
        ('core', '0009_sectionpage_image'),
    ]

    operations = [
        migrations.AddField(
            model_name='homepage',
            name='banner',
            field=models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.SET_NULL, blank=True, to='wagtailimages.Image', null=True),
        ),
        migrations.AddField(
            model_name='homepage',
            name='banner_link_page',
            field=models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.SET_NULL, blank=True, to='wagtailcore.Page', help_text='Optional page to which the banner will link to', null=True),
        ),
    ]



from uuid import uuid4
from django.contrib.auth import hashers
from django.contrib.auth.models import User
from django.core import validators
from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _


from molo.core.models import (
    TranslatablePageMixin, PreventDeleteMixin, Main, index_pages_after_copy)
from molo.core.molo_wagtail_models import MoloPage
from molo.core.utils import generate_slug
from phonenumber_field.modelfields import PhoneNumberField
from wagtail.wagtailcore.models import Site
from wagtail.contrib.settings.models import BaseSetting, register_setting
from wagtail.wagtailadmin.edit_handlers import (
    FieldPanel, MultiFieldPanel, PageChooserPanel)


@register_setting
class UserProfilesSettings(BaseSetting):
    show_mobile_number_field = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Add mobile number field to registration"),
    )
    mobile_number_required = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Mobile number required"),
    )
    country_code = models.CharField(
        max_length=4,
        null=True, blank=True,
        verbose_name=_(
            "The country code that should be added to a user's number for "
            "this site"),
        help_text=_("For example: +27 for South Africa, +44 for England")
    )
    prevent_phone_number_in_username = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Prevent phone number in username / display name"),
    )

    show_email_field = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Add email field to registration")
    )
    email_required = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Email required")
    )

    prevent_email_in_username = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Prevent email in username / display name"),
    )

    show_security_question_fields = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Add security question fields to registration")
    )
    security_questions_required = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Security questions required")
    )
    num_security_questions = models.PositiveSmallIntegerField(
        default=1,
        verbose_name=_("Number of security questions asked for "
                       "password recovery")
    )
    password_recovery_retries = models.PositiveSmallIntegerField(
        default=5,
        verbose_name=_("Max number of password recovery retries before "
                       "lockout")
    )
    terms_and_conditions = models.ForeignKey(
        'wagtailcore.Page',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        help_text=_('Choose a footer page')
    )
    activate_display_name = models.BooleanField(
        default=True,
        editable=True,
        verbose_name=_("Activate Display Name"),
    )
    capture_display_name_on_reg = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Capture On Registration"),
        help_text=_("If Display Name is activated, "
                    "and Capture On Registration is not activated, "
                    "The display name field will be captured on done page."),
    )
    display_name_required = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Display Name Required"),
    )
    activate_gender = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Activate Gender"),
    )
    capture_gender_on_reg = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Capture On Registration"),
        help_text=_("If Gender is activated, "
                    "and Capture On Registration is not activated, "
                    "The Gender field will be captured on done page."),
    )
    gender_required = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Gender Required"),
    )
    activate_dob = models.BooleanField(
        default=True,
        editable=True,
        verbose_name=_("Activate Date Of Birth"),
    )
    capture_dob_on_reg = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Capture On Registration"),
        help_text=_("If Date Of Birth is activated, "
                    "and Capture On Registration is not activated, "
                    "The Date Of Birth field will be captured on done page."),
    )
    dob_required = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Date Of Birth Required"),
    )
    activate_location = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Activate Location"),
    )
    capture_location_on_reg = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Capture On Registration"),
        help_text=_("If Location is activated, "
                    "and Capture On Registration is not activated, "
                    "The Location field will be captured on done page."),
    )
    location_required = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Location Required"),
    )
    activate_education_level = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Activate Education Level"),
    )
    capture_education_level_on_reg = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Capture On Registration"),
        help_text=_("If Education Level is activated, "
                    "and Capture On Registration is not activated, "
                    "The Education Level field will be captured "
                    "on done page."),
    )
    activate_education_level_required = models.BooleanField(
        default=False,
        editable=True,
        verbose_name=_("Education Level Required"),
    )

    panels = [
        MultiFieldPanel(
            [
                FieldPanel('show_mobile_number_field'),
                FieldPanel('mobile_number_required'),
                FieldPanel('country_code'),
                FieldPanel('prevent_phone_number_in_username'),
            ],
            heading="Mobile Number Settings", ),
        MultiFieldPanel(
            [
                FieldPanel('show_email_field'),
                FieldPanel('email_required'),
                FieldPanel('prevent_email_in_username'),
            ],
            heading="Email Settings", ),
        MultiFieldPanel(
            [
                FieldPanel("show_security_question_fields"),
                FieldPanel("security_questions_required"),
                FieldPanel("num_security_questions"),
                FieldPanel("password_recovery_retries"),
            ],
            heading="Security Question Settings", ),
        MultiFieldPanel(
            [
                PageChooserPanel('terms_and_conditions'),
            ],
            heading="Terms and Conditions on registration", ),
        MultiFieldPanel(
            [
                FieldPanel('activate_display_name'),
                FieldPanel('capture_display_name_on_reg'),
                FieldPanel('display_name_required'),
            ],
            heading="Display Name", ),
        MultiFieldPanel(
            [
                FieldPanel('activate_gender'),
                FieldPanel('capture_gender_on_reg'),
                FieldPanel('gender_required'),
            ],
            heading="Gender", ),
        MultiFieldPanel(
            [
                FieldPanel('activate_dob'),
                FieldPanel('capture_dob_on_reg'),
                FieldPanel('dob_required'),
            ],
            heading="Date Of Birth", ),
        MultiFieldPanel(
            [
                FieldPanel('activate_location'),
                FieldPanel('capture_location_on_reg'),
                FieldPanel('location_required'),
            ],
            heading="Location", ),
        MultiFieldPanel(
            [
                FieldPanel('activate_education_level'),
                FieldPanel('capture_education_level_on_reg'),
                FieldPanel('activate_education_level_required'),
            ],
            heading="Education Level", )
    ]
    # TODO: mobile_number_required field shouldn't be shown
    # if show_mobile_number_field is False


@python_2_unicode_compatible
class SecurityQuestion(TranslatablePageMixin, MoloPage):
    parent_page_types = ['SecurityQuestionIndexPage']
    subpage_types = []

    class Meta:
        verbose_name = _("Security Question")

    def __str__(self):
        return self.title


SecurityQuestion.content_panels = [
    FieldPanel("title", classname="full title")
]
SecurityQuestion.promote_panels = []
SecurityQuestion.settings_panels = []


class SecurityQuestionIndexPage(MoloPage, PreventDeleteMixin):
    parent_page_types = ['core.Main']
    subpage_types = ["SecurityQuestion"]

    def copy(self, *args, **kwargs):
        site = kwargs['to'].get_site()
        main = site.root_page
        SecurityQuestionIndexPage.objects.child_of(main).delete()
        super(SecurityQuestionIndexPage, self).copy(*args, **kwargs)


SecurityQuestionIndexPage.content_panels = [
    FieldPanel('title', classname='full title'),
]


@receiver(index_pages_after_copy, sender=Main)
def create_security_question_index_page(sender, instance, **kwargs):
    if not instance.get_children().filter(
            title='Security Questions').exists():
        security_index = SecurityQuestionIndexPage(
            title='Security Questions', slug=('security-questions-%s' % (
                generate_slug(instance.title), )))
        instance.add_child(instance=security_index)
        security_index.save_revision().publish()


class UserProfile(models.Model):
    user = models.OneToOneField(User, related_name="profile", primary_key=True)
    uuid = models.UUIDField(default=uuid4, unique=True)
    auth_service_uuid = models.UUIDField(unique=True, null=True)
    date_of_birth = models.DateField(null=True)
    alias = models.CharField(
        max_length=128,
        blank=True,
        null=True)
    gender = models.CharField(
        max_length=128,
        blank=True,
        null=True)
    location = models.CharField(
        max_length=128,
        blank=True,
        null=True)
    education_level = models.CharField(
        max_length=128,
        blank=True,
        null=True)
    avatar = models.ImageField(
        'avatar',
        max_length=100,
        upload_to='users/profile',
        blank=True,
        null=True)

    mobile_number = PhoneNumberField(blank=True, null=True, unique=False)
    security_question_answers = models.ManyToManyField(
        SecurityQuestion,
        through="SecurityAnswer"
    )
    site = models.ForeignKey(Site, blank=True, null=True)
    admin_sites = models.ManyToManyField(
        Site, related_name='admin_sites'
    )
    migrated_username = models.CharField(
        _('migrated_username'),
        max_length=40,
        validators=[
            validators.RegexValidator(
                r'^[\w.@+-]+$',
                _('Enter a valid username. This value may contain only '
                  'letters, numbers ' 'and @/./+/-/_ characters.')
            ),
        ],
        null=True, blank=True
    )
    fcm_registration_token = models.CharField(max_length=256, null=True)

    panels = [
        FieldPanel('admin_sites',),
    ]


@receiver(post_save, sender=User)
def user_profile_handler(sender, instance, created, **kwargs):
    if created:
        profile = UserProfile(user=instance)
        profile.site = Site.objects.get(is_default_site=True)
        profile.save()


class SecurityAnswer(models.Model):
    user = models.ForeignKey(UserProfile)
    question = models.ForeignKey(SecurityQuestion)
    answer = models.CharField(max_length=150, null=False, blank=False)

    def set_answer(self, raw_answer):
        self.answer = hashers.make_password(raw_answer.strip().lower())

    def check_answer(self, raw_answer):
        def setter(raw_answer):
            self.set_answer(raw_answer)
            self.save(update_fields=["answer"])

        return hashers.check_password(
            raw_answer.strip().lower(),
            self.answer,
            setter
        )

    def save(self, is_import=False, *args, **kwargs):
        # checks if this save is coming from an import so we don't hash a hash
        if not is_import and not self.id:
            self.set_answer(self.answer)
        super(SecurityAnswer, self).save(*args, **kwargs)



from django.test import TestCase

from molo.core.models import (
    ArticlePage,
    Languages,
    PageTranslation,
    SiteLanguageRelation,
)
from molo.core.tests.base import MoloTestCaseMixin


class TestHookDeletePageTranslations(TestCase, MoloTestCaseMixin):
    def setUp(self):
        self.mk_main()

        language_setting = Languages.for_site(self.main.get_site())

        SiteLanguageRelation.objects.create(
            language_setting=language_setting,
            locale='en',
        )

        zulu = SiteLanguageRelation.objects.create(
            language_setting=language_setting,
            locale='zu',
        )

        xhosa = SiteLanguageRelation.objects.create(
            language_setting=language_setting,
            locale='xh',
        )

        self.login()

        self.page_english = self.mk_article(self.main, title='En')

        self.page_xhosa = self.mk_article_translation(
            self.page_english, xhosa, title='Xh')
        self.page_zulu = self.mk_article_translation(
            self.page_english, zulu, title='Zu')

    def test_does_not_delete_on_get_request(self):
        response = self.client.get(
            '/admin/pages/{0}/delete/'.format(self.page_english.id))

        self.assertEqual(response.status_code, 200)
        self.assertEqual(ArticlePage.objects.filter(title='En').count(), 1)
        self.assertEqual(ArticlePage.objects.filter(title='Xh').count(), 1)
        self.assertEqual(ArticlePage.objects.filter(title='Zu').count(), 1)
        self.assertEqual(PageTranslation.objects.all().count(), 2)

    def test_deletes_all_on_post_request(self):
        response = self.client.post(
            '/admin/pages/{0}/delete/'.format(self.page_english.id))

        self.assertEqual(response.status_code, 302)
        self.assertEqual(ArticlePage.objects.filter(title='En').count(), 0)
        self.assertEqual(ArticlePage.objects.filter(title='Xh').count(), 0)
        self.assertEqual(ArticlePage.objects.filter(title='Zu').count(), 0)
        self.assertEqual(PageTranslation.objects.all().count(), 0)

    def test_delete_translation_does_not_delete_original(self):
        response = self.client.post(
            '/admin/pages/{0}/delete/'.format(self.page_zulu.id))

        self.assertEqual(response.status_code, 302)
        self.assertEqual(ArticlePage.objects.filter(title='En').count(), 1)
        self.assertEqual(ArticlePage.objects.filter(title='Xh').count(), 1)
        self.assertEqual(ArticlePage.objects.filter(title='Zu').count(), 0)
        self.assertEqual(PageTranslation.objects.all().count(), 1)
